/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#include <QApplication>
#include <QFrame>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QSpacerItem>
#include <QStringList>

#include "adddialog.h"

TAddDialog::TAddDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFrame								*separator_hline;
	QGridLayout							*dialog_layout;
	QHBoxLayout							*button_hlayout;
	QPushButton							*cancel_button;
	QSpacerItem							*button_hspacer;
	QSpacerItem							*dialog_vspacer;

	this->resize(288,142);

	dialog_layout = new QGridLayout(this);
	
	d_name_label = new QLabel(this);
	dialog_layout->addWidget(d_name_label,0,0,1,1);
	
	d_name_edit = new QLineEdit(this);
	dialog_layout->addWidget(d_name_edit,0,1,1,1);
	
	separator_hline = new QFrame(this);
	separator_hline->setFrameShape(QFrame::HLine);
	separator_hline->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline,1,0,1,2);
	
	dialog_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	dialog_layout->addItem(dialog_vspacer,2,0,1,2);
	
	button_hlayout = new QHBoxLayout();
	
	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);
	
	cancel_button = new QPushButton(this);
	cancel_button->setAutoDefault(false);
	button_hlayout->addWidget(cancel_button);
	
	d_add_button = new QPushButton(this);
	d_add_button->setDefault(true);
	button_hlayout->addWidget(d_add_button);
	dialog_layout->addLayout(button_hlayout,3,0,1,2);
	
	// defaults
	d_add_button->setEnabled(false);

	this->setWindowTitle(QStringLiteral("Add Item"));
	
	cancel_button->setText(QStringLiteral("Cancel"));
	d_add_button->setText(QStringLiteral("Add"));
	d_name_label->setText(QStringLiteral("Name:"));
	
	connect(d_name_edit,&QLineEdit::textEdited,this,&TAddDialog::Name_Changed);
	
	connect(d_add_button,&QPushButton::clicked,this,&TAddDialog::accept);
	connect(cancel_button,&QPushButton::clicked,this,&TAddDialog::reject);
}

TAddDialog::~TAddDialog(void)
{
}

QString TAddDialog::Name(void) const
{
	return d_name_edit->text();
}

void TAddDialog::Reset(
	const TType							type,
	const QStringList					&existing_names)
{
	d_add_button->setEnabled(false);
	d_name_edit->setText(QString());
	
	d_current_type = type;
	d_existing_names = existing_names;

	switch(type)
	{
		case TAddDialog::MACHINE:
			this->setWindowTitle(QStringLiteral("Add Machine"));
			d_name_label->setText(QStringLiteral("Machine Name:"));
			break;
			
		case TAddDialog::MEASUREMENT:
			this->setWindowTitle(QStringLiteral("Add Measurement"));
			d_name_label->setText(QStringLiteral("Measurement Name:"));
			break;
	}
}

void TAddDialog::Name_Changed(
	const QString						&text)
{
	if(text.length() == 0)
	{
		d_add_button->setEnabled(false);
		return;
	}
	
	if(d_existing_names.contains(text))
	{
		d_add_button->setEnabled(false);
		return;
	}

	d_add_button->setEnabled(true);
}

