/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef DISPLAYWIDGETHEADERFILE
#define DISPLAYWIDGETHEADERFILE

#include <QWidget>

#include "../../core/vector3.h"

class QPlainTextEdit;
class QPushButton;
class QTabWidget;
class QLineEdit;
class TOpenGLDisplayWidget;

class TMeasurement;

class TDisplayWidget : public QWidget
{
Q_OBJECT

public:

// CREATORS
	TDisplayWidget(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	~TDisplayWidget(void);

// ACCESSORS
	double Map_Data_Increment_X(void) const;
	double Map_Data_Increment_Y(void) const;
	double Map_Data_Increment_Z(void) const;

// MANIPULATORS
	void Set_Data_Path(const QString &s) {d_data_path = s;}
	
	void Clear_Map_Data(void);
	void Add_Map_Data_Line(const QString &line);
	void Add_Map_Data_End(void);
	
	void Clear_Measurement_Data(void);
	void Add_Measurement_Data_Line(const QString &line);
	void Add_Measurement_Data_End(void);
	
	void Clear_Graphical_Data(void);
	void Add_Graphical_Data(TMeasurement * const measurement);
	void Add_Graphical_Data_End(void);
	
	void Clear_Selection_Data(void);
	void Add_Selection_Data_Line(const QString &line);
	void Add_Selection_Data_End(void);
	
	void Enable_Output_Options(const bool state);
	
	void Define_Machine_Volume(const TVector3 &min_corner,const TVector3 &max_corner);
	void Clear_Machine_Volume(void);

signals:
	void Item_Selected(const int,const int);	// class and measurement item index
	void Export_Measurements(void);
	void Evaluate_Measurements(void);
	void Update_Map_Display(void);

private slots:
	void Measurement_Item_Clicked(const double&,const double&,const double&,const int);
	void Save_Measurement_Text(void);

private:
	QPlainTextEdit						*d_selection_edit;
	QPlainTextEdit						*d_map_data_edit;
	QPlainTextEdit						*d_measurement_edit;
	QPushButton							*d_save_text_button;
	QPushButton							*d_export_button;
	QPushButton							*d_evaluate_button;
	QPushButton							*d_update_map_button;
	QTabWidget							*d_display_tab;
	TOpenGLDisplayWidget				*d_opengl_display_widget;
	QLineEdit							*d_map_increment_x_edit;
	QLineEdit							*d_map_increment_y_edit;
	QLineEdit							*d_map_increment_z_edit;
	
	QString								d_data_path;

// NOT IMPLEMENTED
	TDisplayWidget(const TDisplayWidget&);
	TDisplayWidget& operator=(const TDisplayWidget&);
};

#endif
