/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSpacerItem>
#include <QToolButton>
#include <QCheckBox>
#include <assert.h>

#include "editor_ballbar.h"

TEditor_Ballbar::TEditor_Ballbar(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QWidget(const_cast<QWidget*>(parent),flags)
{
	QGridLayout							*widget_layout;
	QLabel								*center_position_label;
	QLabel								*center_position_x_label;
	QLabel								*center_position_y_label;
	QLabel								*center_position_z_label;
	QLabel								*direction_vector_i_label;
	QLabel								*direction_vector_j_label;
	QLabel								*direction_vector_k_label;
	QLabel								*direction_vector_label;
	QLabel								*name_label;
	QLabel								*editor_probe_x_label;
	QLabel								*editor_probe_y_label;
	QLabel								*editor_probe_z_label;
	QLabel								*nominal_length_label;
	QLabel								*probe_offset_label;
	QSpacerItem							*widget_vspacer;

	widget_layout = new QGridLayout(this);
	
	name_label = new QLabel(this);
	widget_layout->addWidget(name_label,0,0,1,1);
	
	d_name_edit = new QLineEdit(this);
	d_name_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_name_edit,0,2,1,1);

	nominal_length_label = new QLabel(this);
	widget_layout->addWidget(nominal_length_label,1,0,1,1);

	d_nominal_length_edit = new QLineEdit(this);
	d_nominal_length_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_nominal_length_edit,1,2,1,1);

	center_position_label = new QLabel(this);
	widget_layout->addWidget(center_position_label,2,0,1,1);

	center_position_x_label = new QLabel(this);
	center_position_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(center_position_x_label,2,1,1,1);

	d_editor_position_x_edit = new QLineEdit(this);
	d_editor_position_x_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_position_x_edit,2,2,1,1);

	center_position_y_label = new QLabel(this);
	center_position_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(center_position_y_label,3,1,1,1);

	d_editor_position_y_edit = new QLineEdit(this);
	d_editor_position_y_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_position_y_edit,3,2,1,1);

	center_position_z_label = new QLabel(this);
	center_position_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(center_position_z_label,4,1,1,1);

	d_editor_position_z_edit = new QLineEdit(this);
	d_editor_position_z_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_position_z_edit,4,2,1,1);

	direction_vector_label = new QLabel(this);
	widget_layout->addWidget(direction_vector_label,5,0,1,1);

	direction_vector_i_label = new QLabel(this);
	direction_vector_i_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(direction_vector_i_label,5,1,1,1);

	d_editor_direction_i_edit = new QLineEdit(this);
	d_editor_direction_i_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_direction_i_edit,5,2,1,1);

	direction_vector_j_label = new QLabel(this);
	direction_vector_j_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(direction_vector_j_label,6,1,1,1);

	d_editor_direction_j_edit = new QLineEdit(this);
	d_editor_direction_j_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_direction_j_edit,6,2,1,1);

	direction_vector_k_label = new QLabel(this);
	direction_vector_k_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(direction_vector_k_label,7,1,1,1);

	d_editor_direction_k_edit = new QLineEdit(this);
	d_editor_direction_k_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_direction_k_edit,7,2,1,1);

	probe_offset_label = new QLabel(this);
	widget_layout->addWidget(probe_offset_label,8,0,1,1);

	editor_probe_x_label = new QLabel(this);
	editor_probe_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(editor_probe_x_label,8,1,1,1);

	d_editor_probe_x_edit = new QLineEdit(this);
	d_editor_probe_x_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_probe_x_edit,8,2,1,1);

	editor_probe_y_label = new QLabel(this);
	editor_probe_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(editor_probe_y_label,9,1,1,1);

	d_editor_probe_y_edit = new QLineEdit(this);
	d_editor_probe_y_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_probe_y_edit,9,2,1,1);

	editor_probe_z_label = new QLabel(this);
	editor_probe_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(editor_probe_z_label,10,1,1,1);

	d_editor_probe_z_edit = new QLineEdit(this);
	d_editor_probe_z_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_probe_z_edit,10,2,1,1);

	widget_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	widget_layout->addItem(widget_vspacer,11,0,1,3);

	QWidget::setTabOrder(d_name_edit,d_nominal_length_edit);
	QWidget::setTabOrder(d_nominal_length_edit,d_editor_position_x_edit);
	QWidget::setTabOrder(d_editor_position_x_edit,d_editor_position_y_edit);
	QWidget::setTabOrder(d_editor_position_y_edit,d_editor_position_z_edit);
	QWidget::setTabOrder(d_editor_position_z_edit,d_editor_direction_i_edit);
	QWidget::setTabOrder(d_editor_direction_i_edit,d_editor_direction_j_edit);
	QWidget::setTabOrder(d_editor_direction_j_edit,d_editor_direction_k_edit);
	QWidget::setTabOrder(d_editor_direction_k_edit,d_editor_probe_x_edit);
	QWidget::setTabOrder(d_editor_probe_x_edit,d_editor_probe_y_edit);
	QWidget::setTabOrder(d_editor_probe_y_edit,d_editor_probe_z_edit);

	nominal_length_label->setText(QStringLiteral("Length:"));
	name_label->setText(QStringLiteral("Name:"));
	center_position_label->setText(QStringLiteral("Center Position:"));
	center_position_x_label->setText(QStringLiteral("X:"));
	center_position_y_label->setText(QStringLiteral("Y:"));
	center_position_z_label->setText(QStringLiteral("Z:"));
	direction_vector_label->setText(QStringLiteral("Direction Vector:"));
	direction_vector_i_label->setText(QStringLiteral("I:"));
	direction_vector_j_label->setText(QStringLiteral("J:"));
	direction_vector_k_label->setText(QStringLiteral("K:"));
	probe_offset_label->setText(QStringLiteral("Probe Offset:"));
	editor_probe_x_label->setText(QStringLiteral("X:"));
	editor_probe_y_label->setText(QStringLiteral("Y:"));
	editor_probe_z_label->setText(QStringLiteral("Z:"));
	
	// defaults
	d_name_edit->setText(QStringLiteral("Ballbar 1"));
	d_editor_position_x_edit->setText(QStringLiteral("XMid"));
	d_editor_position_y_edit->setText(QStringLiteral("YMid"));
	d_editor_position_z_edit->setText(QStringLiteral("ZMid"));
	d_editor_direction_i_edit->setText(QStringLiteral("1.000"));
	d_editor_direction_j_edit->setText(QStringLiteral("0.000"));
	d_editor_direction_k_edit->setText(QStringLiteral("0.000"));
	d_nominal_length_edit->setText(QStringLiteral("AxisMin - 100"));
	d_editor_probe_x_edit->setText(QStringLiteral("0.000"));
	d_editor_probe_y_edit->setText(QStringLiteral("0.000"));
	d_editor_probe_z_edit->setText(QStringLiteral("-75.000"));

	connect(d_editor_direction_i_edit,&QLineEdit::textEdited,this,&TEditor_Ballbar::Data_Changed);
	connect(d_editor_direction_j_edit,&QLineEdit::textEdited,this,&TEditor_Ballbar::Data_Changed);
	connect(d_editor_direction_k_edit,&QLineEdit::textEdited,this,&TEditor_Ballbar::Data_Changed);
	connect(d_editor_position_x_edit,&QLineEdit::textEdited,this,&TEditor_Ballbar::Data_Changed);
	connect(d_editor_position_y_edit,&QLineEdit::textEdited,this,&TEditor_Ballbar::Data_Changed);
	connect(d_editor_position_z_edit,&QLineEdit::textEdited,this,&TEditor_Ballbar::Data_Changed);
	connect(d_editor_probe_x_edit,&QLineEdit::textEdited,this,&TEditor_Ballbar::Data_Changed);
	connect(d_editor_probe_y_edit,&QLineEdit::textEdited,this,&TEditor_Ballbar::Data_Changed);
	connect(d_editor_probe_z_edit,&QLineEdit::textEdited,this,&TEditor_Ballbar::Data_Changed);
	connect(d_nominal_length_edit,&QLineEdit::textEdited,this,&TEditor_Ballbar::Data_Changed);
}

TEditor_Ballbar::~TEditor_Ballbar(void)
{
}

QString TEditor_Ballbar::Name(void) const
{
	return d_name_edit->text();
}

QString TEditor_Ballbar::Position_X(void) const
{
	return d_editor_position_x_edit->text();
}

QString TEditor_Ballbar::Position_Y(void) const
{
	return d_editor_position_y_edit->text();
}

QString TEditor_Ballbar::Position_Z(void) const
{
	return d_editor_position_z_edit->text();
}

QString TEditor_Ballbar::Direction_I(void) const
{
	return d_editor_direction_i_edit->text();
}

QString TEditor_Ballbar::Direction_J(void) const
{
	return d_editor_direction_j_edit->text();
}

QString TEditor_Ballbar::Direction_K(void) const
{
	return d_editor_direction_k_edit->text();
}

TVector3 TEditor_Ballbar::Probe_Offset(void) const
{
	TVector3							offset;
	
	offset.x = d_editor_probe_x_edit->text().toDouble();
	offset.y = d_editor_probe_y_edit->text().toDouble();
	offset.z = d_editor_probe_z_edit->text().toDouble();
	
	return offset;
}

QString TEditor_Ballbar::Length(void) const
{
	return d_nominal_length_edit->text();
}

void TEditor_Ballbar::Set_Name(
	const QString						&s)
{
	d_name_edit->setText(s);
}

void TEditor_Ballbar::Enable_Name(
	const bool							state)
{
	d_name_edit->setEnabled(state);
}

void TEditor_Ballbar::Set_Position_X(
	const QString						&s)
{
	d_editor_position_x_edit->setText(s);
}

void TEditor_Ballbar::Set_Position_Y(
	const QString						&s)
{
	d_editor_position_y_edit->setText(s);
}

void TEditor_Ballbar::Set_Position_Z(
	const QString						&s)
{
	d_editor_position_z_edit->setText(s);
}

void TEditor_Ballbar::Set_Direction_I(
	const QString						&s)
{
	d_editor_direction_i_edit->setText(s);
}

void TEditor_Ballbar::Set_Direction_J(
	const QString						&s)
{
	d_editor_direction_j_edit->setText(s);
}

void TEditor_Ballbar::Set_Direction_K(
	const QString						&s)
{
	d_editor_direction_k_edit->setText(s);
}
	
void TEditor_Ballbar::Set_Probe_Offset(
	const TVector3 						&offset)
{
	d_editor_probe_x_edit->setText(QString("%1").arg(offset.x,0,'f',4));
	d_editor_probe_y_edit->setText(QString("%1").arg(offset.y,0,'f',4));
	d_editor_probe_z_edit->setText(QString("%1").arg(offset.z,0,'f',4));	
}

void TEditor_Ballbar::Set_Length(
	const QString						&s)
{
	d_nominal_length_edit->setText(s);
}


