/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef EDITOR_BALLBARHEADERFILE
#define EDITOR_BALLBARHEADERFILE

#include <QString>
#include <QWidget>

#include "../../core/vector3.h"

class QLineEdit;

class TEditor_Ballbar : public QWidget
{
Q_OBJECT

public:

// CREATORS
	TEditor_Ballbar(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	~TEditor_Ballbar(void);

// ACCESSORS
	QString Name(void) const;

	QString Position_X(void) const;
	QString Position_Y(void) const;
	QString Position_Z(void) const;
	
	QString Direction_I(void) const;
	QString Direction_J(void) const;
	QString Direction_K(void) const;
	
	TVector3 Probe_Offset(void) const;
	QString Length(void) const;

// MANIPULATORS
	void Set_Name(const QString &s);
	void Enable_Name(const bool state);

	void Set_Position_X(const QString &s);
	void Set_Position_Y(const QString &s);
	void Set_Position_Z(const QString &s);

	void Set_Direction_I(const QString &s);
	void Set_Direction_J(const QString &s);
	void Set_Direction_K(const QString &s);

	void Set_Probe_Offset(const TVector3 &offset);
	void Set_Length(const QString &s);

signals:
	void Data_Changed(void);

private slots:

private:
	QLineEdit							*d_name_edit;
	QLineEdit							*d_editor_position_x_edit;
	QLineEdit							*d_editor_position_y_edit;
	QLineEdit							*d_editor_position_z_edit;
	QLineEdit							*d_editor_direction_i_edit;
	QLineEdit							*d_editor_direction_j_edit;
	QLineEdit							*d_editor_direction_k_edit;
	QLineEdit							*d_editor_probe_x_edit;
	QLineEdit							*d_editor_probe_y_edit;
	QLineEdit							*d_editor_probe_z_edit;
	QLineEdit							*d_nominal_length_edit;

// NOT IMPLEMENTED
	TEditor_Ballbar(const TEditor_Ballbar&);
	TEditor_Ballbar& operator=(const TEditor_Ballbar&);
};

#endif
