/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSpacerItem>
#include <QToolButton>

#include "editor_iso.h"

TEditor_ISO::TEditor_ISO(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QWidget(const_cast<QWidget*>(parent),flags)
{
	QGridLayout							*widget_layout;
	QLabel								*end_position_label;
	QLabel								*end_position_x_label;
	QLabel								*end_position_y_label;
	QLabel								*end_position_z_label;
	QLabel								*name_label;
	QLabel								*probe_offset_label;
	QLabel								*probe_offset_x_label;
	QLabel								*probe_offset_y_label;
	QLabel								*probe_offset_z_label;
	QLabel								*start_position_label;
	QLabel								*start_position_x_label;
	QLabel								*start_position_y_label;
	QLabel								*start_position_z_label;
	QSpacerItem							*widget_vspacer;

	widget_layout = new QGridLayout(this);

	name_label = new QLabel(this);
	widget_layout->addWidget(name_label,0,0,1,1);

	d_name_edit = new QLineEdit(this);
	d_name_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_name_edit,0,2,1,1);

	start_position_label = new QLabel(this);
	widget_layout->addWidget(start_position_label,1,0,1,1);

	start_position_x_label = new QLabel(this);
	start_position_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(start_position_x_label,1,1,1,1);

	d_editor_start_position_x_edit = new QLineEdit(this);
	d_editor_start_position_x_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_start_position_x_edit,1,2,1,1);

	start_position_y_label = new QLabel(this);
	start_position_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(start_position_y_label,2,1,1,1);

	d_editor_start_position_y_edit = new QLineEdit(this);
	d_editor_start_position_y_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_start_position_y_edit,2,2,1,1);

	start_position_z_label = new QLabel(this);
	start_position_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(start_position_z_label,3,1,1,1);

	d_editor_start_position_z_edit = new QLineEdit(this);
	d_editor_start_position_z_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_start_position_z_edit,3,2,1,1);

	end_position_label = new QLabel(this);
	widget_layout->addWidget(end_position_label,4,0,1,1);

	end_position_x_label = new QLabel(this);
	end_position_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(end_position_x_label,4,1,1,1);

	d_editor_end_position_x_edit = new QLineEdit(this);
	d_editor_end_position_x_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_end_position_x_edit,4,2,1,1);

	end_position_y_label = new QLabel(this);
	end_position_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(end_position_y_label,5,1,1,1);

	d_editor_end_position_y_edit = new QLineEdit(this);
	d_editor_end_position_y_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_end_position_y_edit,5,2,1,1);

	end_position_z_label = new QLabel(this);
	end_position_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(end_position_z_label,6,1,1,1);

	d_editor_end_position_z_edit = new QLineEdit(this);
	d_editor_end_position_z_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_end_position_z_edit,6,2,1,1);

	probe_offset_label = new QLabel(this);
	widget_layout->addWidget(probe_offset_label,7,0,1,1);

	probe_offset_x_label = new QLabel(this);
	probe_offset_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(probe_offset_x_label,7,1,1,1);

	d_editor_probe_x_edit = new QLineEdit(this);
	d_editor_probe_x_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_probe_x_edit,7,2,1,1);

	probe_offset_y_label = new QLabel(this);
	probe_offset_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(probe_offset_y_label,8,1,1,1);

	d_editor_probe_y_edit = new QLineEdit(this);
	d_editor_probe_y_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_probe_y_edit,8,2,1,1);

	probe_offset_z_label = new QLabel(this);
	probe_offset_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(probe_offset_z_label,9,1,1,1);

	d_editor_probe_z_edit = new QLineEdit(this);
	d_editor_probe_z_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_probe_z_edit,9,2,1,1);

	widget_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	widget_layout->addItem(widget_vspacer,11,0,1,3);

	QWidget::setTabOrder(d_name_edit,d_editor_start_position_x_edit);
	QWidget::setTabOrder(d_editor_start_position_x_edit,d_editor_start_position_y_edit);
	QWidget::setTabOrder(d_editor_start_position_y_edit,d_editor_start_position_z_edit);
	QWidget::setTabOrder(d_editor_start_position_z_edit,d_editor_end_position_x_edit);
	QWidget::setTabOrder(d_editor_end_position_x_edit,d_editor_end_position_y_edit);
	QWidget::setTabOrder(d_editor_end_position_y_edit,d_editor_end_position_z_edit);
	QWidget::setTabOrder(d_editor_end_position_z_edit,d_editor_probe_x_edit);
	QWidget::setTabOrder(d_editor_probe_x_edit,d_editor_probe_y_edit);
	QWidget::setTabOrder(d_editor_probe_y_edit,d_editor_probe_z_edit);

	name_label->setText(QStringLiteral("Name:"));
	start_position_label->setText(QStringLiteral("Start Position:"));
	start_position_x_label->setText(QStringLiteral("X:"));
	start_position_y_label->setText(QStringLiteral("Y:"));
	start_position_z_label->setText(QStringLiteral("Z:"));
	end_position_label->setText(QStringLiteral("End Position"));
	end_position_x_label->setText(QStringLiteral("X:"));
	end_position_y_label->setText(QStringLiteral("Y:"));
	end_position_z_label->setText(QStringLiteral("Z:"));
	probe_offset_label->setText(QStringLiteral("Probe Offset:"));
	probe_offset_x_label->setText(QStringLiteral("X:"));
	probe_offset_y_label->setText(QStringLiteral("Y:"));
	probe_offset_z_label->setText(QStringLiteral("Z:"));

	// defaults
	d_name_edit->setText(QStringLiteral("ISO 1"));
	d_editor_start_position_x_edit->setText(QStringLiteral("XMin"));
	d_editor_start_position_y_edit->setText(QStringLiteral("YMin"));
	d_editor_start_position_z_edit->setText(QStringLiteral("ZMin"));
	d_editor_end_position_x_edit->setText(QStringLiteral("XMax"));
	d_editor_end_position_y_edit->setText(QStringLiteral("YMax"));
	d_editor_end_position_z_edit->setText(QStringLiteral("ZMax"));
	d_editor_probe_x_edit->setText(QStringLiteral("0.000"));
	d_editor_probe_y_edit->setText(QStringLiteral("0.000"));
	d_editor_probe_z_edit->setText(QStringLiteral("-75.000"));
	
	connect(d_editor_end_position_x_edit,&QLineEdit::textEdited,this,&TEditor_ISO::Data_Changed);
	connect(d_editor_end_position_y_edit,&QLineEdit::textEdited,this,&TEditor_ISO::Data_Changed);
	connect(d_editor_end_position_z_edit,&QLineEdit::textEdited,this,&TEditor_ISO::Data_Changed);
	connect(d_editor_start_position_x_edit,&QLineEdit::textEdited,this,&TEditor_ISO::Data_Changed);
	connect(d_editor_start_position_y_edit,&QLineEdit::textEdited,this,&TEditor_ISO::Data_Changed);
	connect(d_editor_start_position_z_edit,&QLineEdit::textEdited,this,&TEditor_ISO::Data_Changed);
	connect(d_editor_probe_x_edit,&QLineEdit::textEdited,this,&TEditor_ISO::Data_Changed);
	connect(d_editor_probe_y_edit,&QLineEdit::textEdited,this,&TEditor_ISO::Data_Changed);
	connect(d_editor_probe_z_edit,&QLineEdit::textEdited,this,&TEditor_ISO::Data_Changed);
}

TEditor_ISO::~TEditor_ISO(void)
{
}

QString TEditor_ISO::Name(void) const
{
	return d_name_edit->text();
}

QString TEditor_ISO::Start_Position_X(void) const
{
	return d_editor_start_position_x_edit->text();
}

QString TEditor_ISO::Start_Position_Y(void) const
{
	return d_editor_start_position_y_edit->text();
}

QString TEditor_ISO::Start_Position_Z(void) const
{
	return d_editor_start_position_z_edit->text();
}

QString TEditor_ISO::End_Position_X(void) const
{
	return d_editor_end_position_x_edit->text();
}

QString TEditor_ISO::End_Position_Y(void) const
{
	return d_editor_end_position_y_edit->text();
}

QString TEditor_ISO::End_Position_Z(void) const
{
	return d_editor_end_position_z_edit->text();
}

TVector3 TEditor_ISO::Probe_Offset(void) const
{
	TVector3							offset;
	
	offset.x = d_editor_probe_x_edit->text().toDouble();
	offset.y = d_editor_probe_y_edit->text().toDouble();
	offset.z = d_editor_probe_z_edit->text().toDouble();
	
	return offset;
}

void TEditor_ISO::Set_Name(const QString &s)
{
	d_name_edit->setText(s);
}

void TEditor_ISO::Enable_Name(
	const bool							state)
{
	d_name_edit->setEnabled(state);
}

void TEditor_ISO::Set_Start_Position_X(
	const QString						&s)
{
	d_editor_start_position_x_edit->setText(s);
}

void TEditor_ISO::Set_Start_Position_Y(
	const QString						&s)
{
	d_editor_start_position_y_edit->setText(s);
}

void TEditor_ISO::Set_Start_Position_Z(
	const QString						&s)
{
	d_editor_start_position_z_edit->setText(s);
}

void TEditor_ISO::Set_End_Position_X(
	const QString						&s)
{
	d_editor_end_position_x_edit->setText(s);
}

void TEditor_ISO::Set_End_Position_Y(
	const QString						&s)
{
	d_editor_end_position_y_edit->setText(s);
}

void TEditor_ISO::Set_End_Position_Z(
	const QString						&s)
{
	d_editor_end_position_z_edit->setText(s);
}
	
void TEditor_ISO::Set_Probe_Offset(
	const TVector3 						&offset)
{
	d_editor_probe_x_edit->setText(QString("%1").arg(offset.x,0,'f',4));
	d_editor_probe_y_edit->setText(QString("%1").arg(offset.y,0,'f',4));
	d_editor_probe_z_edit->setText(QString("%1").arg(offset.z,0,'f',4));	
}
