/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef EDITOR_ISOHEADERFILE
#define EDITOR_ISOHEADERFILE

#include <QWidget>

#include "../../core/vector3.h"

class QLineEdit;

class TEditor_ISO : public QWidget
{
Q_OBJECT

public:

// CREATORS
	TEditor_ISO(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	~TEditor_ISO(void);

// ACCESSORS
	QString Name(void) const;

	QString Start_Position_X(void) const;
	QString Start_Position_Y(void) const;
	QString Start_Position_Z(void) const;
	
	QString End_Position_X(void) const;
	QString End_Position_Y(void) const;
	QString End_Position_Z(void) const;
	
	TVector3 Probe_Offset(void) const;

// MANIPULATORS
	void Set_Name(const QString &s);
	void Enable_Name(const bool state);

	void Set_Start_Position_X(const QString &s);
	void Set_Start_Position_Y(const QString &s);
	void Set_Start_Position_Z(const QString &s);
	
	void Set_End_Position_X(const QString &s);
	void Set_End_Position_Y(const QString &s);
	void Set_End_Position_Z(const QString &s);
	
	void Set_Probe_Offset(const TVector3 &offset);

signals:
	void Data_Changed(void);

private slots:

private:
	QLineEdit							*d_name_edit;
	QLineEdit							*d_editor_start_position_x_edit;
	QLineEdit							*d_editor_start_position_y_edit;
	QLineEdit							*d_editor_start_position_z_edit;
	QLineEdit							*d_editor_end_position_x_edit;
	QLineEdit							*d_editor_end_position_y_edit;
	QLineEdit							*d_editor_end_position_z_edit;
	QLineEdit							*d_editor_probe_x_edit;
	QLineEdit							*d_editor_probe_y_edit;
	QLineEdit							*d_editor_probe_z_edit;

// NOT IMPLEMENTED
	TEditor_ISO(const TEditor_ISO&);
	TEditor_ISO& operator=(const TEditor_ISO&);
};

#endif
