/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSpacerItem>
#include <QToolButton>

#include "editor_mcg.h"

TEditor_MCG::TEditor_MCG(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QWidget(const_cast<QWidget*>(parent),flags)
{
	QGridLayout							*widget_layout;
	QLabel								*azimuth_count_label;
	QLabel								*center_position_label;
	QLabel								*center_position_x_label;
	QLabel								*center_position_y_label;
	QLabel								*center_position_z_label;
	QLabel								*elevation_count_label;
	QLabel								*end_elevation_label;
	QLabel								*length_label;
	QLabel								*name_label;
	QLabel								*probe_offset_label;
	QLabel								*probe_offset_x_label;
	QLabel								*probe_offset_y_label;
	QLabel								*probe_offset_z_label;
	QLabel								*start_elevation_label;
	QSpacerItem							*widget_vspacer;

	widget_layout = new QGridLayout(this);

	name_label = new QLabel(this);
	widget_layout->addWidget(name_label,0,0,1,1);

	d_name_edit = new QLineEdit(this);
	d_name_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_name_edit,0,2,1,1);

	length_label = new QLabel(this);
	widget_layout->addWidget(length_label,1,0,1,1);

	d_length_edit = new QLineEdit(this);
	d_length_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_length_edit,1,2,1,1);

	center_position_label = new QLabel(this);
	widget_layout->addWidget(center_position_label,2,0,1,1);

	center_position_x_label = new QLabel(this);
	center_position_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(center_position_x_label,2,1,1,1);

	d_editor_center_position_x_edit = new QLineEdit(this);
	d_editor_center_position_x_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_center_position_x_edit,2,2,1,1);

	center_position_y_label = new QLabel(this);
	center_position_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(center_position_y_label,3,1,1,1);

	d_editor_center_position_y_edit = new QLineEdit(this);
	d_editor_center_position_y_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_center_position_y_edit,3,2,1,1);

	center_position_z_label = new QLabel(this);
	center_position_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(center_position_z_label,4,1,1,1);

	d_editor_center_position_z_edit = new QLineEdit(this);
	d_editor_center_position_z_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_center_position_z_edit,4,2,1,1);

	start_elevation_label = new QLabel(this);
	widget_layout->addWidget(start_elevation_label,5,0,1,1);

	d_start_elevation_edit = new QLineEdit(this);
	d_start_elevation_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_start_elevation_edit,5,2,1,1);

	end_elevation_label = new QLabel(this);
	widget_layout->addWidget(end_elevation_label,6,0,1,1);

	d_end_elevation_edit = new QLineEdit(this);
	d_end_elevation_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_end_elevation_edit,6,2,1,1);

	elevation_count_label = new QLabel(this);
	widget_layout->addWidget(elevation_count_label,7,0,1,1);

	d_elevation_count_edit = new QLineEdit(this);
	d_elevation_count_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_elevation_count_edit,7,2,1,1);

	azimuth_count_label = new QLabel(this);
	widget_layout->addWidget(azimuth_count_label,8,0,1,1);

	d_azimuth_count_edit = new QLineEdit(this);
	d_azimuth_count_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_azimuth_count_edit,8,2,1,1);

	probe_offset_label = new QLabel(this);
	widget_layout->addWidget(probe_offset_label,9,0,1,1);

	probe_offset_x_label = new QLabel(this);
	probe_offset_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(probe_offset_x_label,9,1,1,1);

	d_editor_probe_x_edit = new QLineEdit(this);
	d_editor_probe_x_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_probe_x_edit,9,2,1,1);

	probe_offset_y_label = new QLabel(this);
	probe_offset_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(probe_offset_y_label,10,1,1,1);

	d_editor_probe_y_edit = new QLineEdit(this);
	d_editor_probe_y_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_probe_y_edit,10,2,1,1);

	probe_offset_z_label = new QLabel(this);
	probe_offset_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(probe_offset_z_label,11,1,1,1);

	d_editor_probe_z_edit = new QLineEdit(this);
	d_editor_probe_z_edit->setAlignment(Qt::AlignLeft);
	widget_layout->addWidget(d_editor_probe_z_edit,11,2,1,1);

	widget_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	widget_layout->addItem(widget_vspacer,12,0,1,3);

	QWidget::setTabOrder(d_name_edit,d_length_edit);
	QWidget::setTabOrder(d_length_edit,d_editor_center_position_x_edit);
	QWidget::setTabOrder(d_editor_center_position_x_edit,d_editor_center_position_y_edit);
	QWidget::setTabOrder(d_editor_center_position_y_edit,d_editor_center_position_z_edit);
	QWidget::setTabOrder(d_editor_center_position_z_edit,d_start_elevation_edit);
	QWidget::setTabOrder(d_start_elevation_edit,d_end_elevation_edit);
	QWidget::setTabOrder(d_end_elevation_edit,d_elevation_count_edit);
	QWidget::setTabOrder(d_elevation_count_edit,d_azimuth_count_edit);
	QWidget::setTabOrder(d_azimuth_count_edit,d_editor_probe_x_edit);
	QWidget::setTabOrder(d_editor_probe_x_edit,d_editor_probe_y_edit);
	QWidget::setTabOrder(d_editor_probe_y_edit,d_editor_probe_z_edit);

	name_label->setText(QStringLiteral("Name:"));
	length_label->setText(QStringLiteral("Arm Length:"));
	center_position_label->setText(QStringLiteral("Center Position:"));
	center_position_x_label->setText(QStringLiteral("X:"));
	center_position_y_label->setText(QStringLiteral("Y:"));
	center_position_z_label->setText(QStringLiteral("Z:"));
	start_elevation_label->setText(QStringLiteral("Start Elevation:"));
	end_elevation_label->setText(QStringLiteral("End Elevation:"));
	elevation_count_label->setText(QStringLiteral("Elevation Count:"));
	azimuth_count_label->setText(QStringLiteral("Azimuth Count:"));
	d_azimuth_count_edit->setText(QStringLiteral(""));
	probe_offset_label->setText(QStringLiteral("Probe Offset:"));
	probe_offset_x_label->setText(QStringLiteral("X:"));
	probe_offset_y_label->setText(QStringLiteral("Y:"));
	probe_offset_z_label->setText(QStringLiteral("Z:"));
	
	// default values
	d_name_edit->setText(QStringLiteral("MCG 1"));
	d_editor_center_position_x_edit->setText(QStringLiteral("XMid"));
	d_editor_center_position_y_edit->setText(QStringLiteral("YMid"));
	d_editor_center_position_z_edit->setText(QStringLiteral("ZMin + 300"));
	d_editor_probe_x_edit->setText(QStringLiteral("0.000"));
	d_editor_probe_y_edit->setText(QStringLiteral("0.000"));
	d_editor_probe_z_edit->setText(QStringLiteral("-75.000"));
	d_length_edit->setText(QStringLiteral("300"));
	d_azimuth_count_edit->setText(QStringLiteral("8"));
	d_elevation_count_edit->setText(QStringLiteral("3"));
	d_start_elevation_edit->setText(QStringLiteral("-45"));
	d_end_elevation_edit->setText(QStringLiteral("45"));
	
	connect(d_azimuth_count_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
	connect(d_elevation_count_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
	connect(d_end_elevation_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
	connect(d_editor_center_position_x_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
	connect(d_editor_center_position_y_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
	connect(d_editor_center_position_z_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
	connect(d_editor_probe_x_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
	connect(d_editor_probe_y_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
	connect(d_editor_probe_z_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
	connect(d_length_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
	connect(d_start_elevation_edit,&QLineEdit::textEdited,this,&TEditor_MCG::Data_Changed);
}

TEditor_MCG::~TEditor_MCG(void)
{
}

QString TEditor_MCG::Name(void) const
{
	return d_name_edit->text();
}

QString TEditor_MCG::Position_X(void) const
{
	return d_editor_center_position_x_edit->text();
}

QString TEditor_MCG::Position_Y(void) const
{
	return d_editor_center_position_y_edit->text();
}

QString TEditor_MCG::Position_Z(void) const
{
	return d_editor_center_position_z_edit->text();
}

TVector3 TEditor_MCG::Probe_Offset(void) const
{
	TVector3							offset;
	
	offset.x = d_editor_probe_x_edit->text().toDouble();
	offset.y = d_editor_probe_y_edit->text().toDouble();
	offset.z = d_editor_probe_z_edit->text().toDouble();
	
	return offset;
}

QString TEditor_MCG::Length(void) const
{
	return d_length_edit->text();
}

QString TEditor_MCG::Azimuth_Count(void) const
{
	return d_azimuth_count_edit->text();
}

QString TEditor_MCG::Elevation_Count(void) const
{
	return d_elevation_count_edit->text();
}

QString TEditor_MCG::Start_Elevation(void) const
{
	return d_start_elevation_edit->text();
}

QString TEditor_MCG::End_Elevation(void) const
{
	return d_end_elevation_edit->text();
}

void TEditor_MCG::Set_Name(
	const QString						&s)
{
	d_name_edit->setText(s);
}

void TEditor_MCG::Enable_Name(
	const bool							state)
{
	d_name_edit->setEnabled(state);
}

void TEditor_MCG::Set_Position_X(
	const QString						&s)
{
	d_editor_center_position_x_edit->setText(s);
}

void TEditor_MCG::Set_Position_Y(
	const QString						&s)
{
	d_editor_center_position_y_edit->setText(s);
}

void TEditor_MCG::Set_Position_Z(
	const QString						&s)
{
	d_editor_center_position_z_edit->setText(s);
}
	
void TEditor_MCG::Set_Probe_Offset(
	const TVector3 						&offset)
{
	d_editor_probe_x_edit->setText(QString("%1").arg(offset.x,0,'f',4));
	d_editor_probe_y_edit->setText(QString("%1").arg(offset.y,0,'f',4));
	d_editor_probe_z_edit->setText(QString("%1").arg(offset.z,0,'f',4));	
}

void TEditor_MCG::Set_Length(
	const QString						&s)
{
	d_length_edit->setText(s);
}

void TEditor_MCG::Set_Azimuth_Count(
	const QString						&s)
{
	d_azimuth_count_edit->setText(s);
}

void TEditor_MCG::Set_Elevation_Count(
	const QString						&s)
{
	d_elevation_count_edit->setText(s);
}

void TEditor_MCG::Set_Start_Elevation(
	const QString						&s)
{
	d_start_elevation_edit->setText(s);
}

void TEditor_MCG::Set_End_Elevation(
	const QString						&s)
{
	d_end_elevation_edit->setText(s);
}

