/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef ERRORSIMULATORHEADERFILE
#define ERRORSIMULATORHEADERFILE

#include <QMainWindow>

#include "machinecomp.h"
#include "machinedialog.h"

class QAction;
class QCloseEvent;
class QListWidget;
class QToolButton;

class TAddDialog;
class TDisplayWidget;
class TMeasurementDialog;
class TMeasurement;
class TProgressDialog;
class TMessageBox;

class TErrorSimulator : public QMainWindow
{
Q_OBJECT

public:

// CREATORS
	TErrorSimulator(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	virtual ~TErrorSimulator(void);

// ACCESSORS

// MANIPULATORS

signals:

private slots:
	void Machine_Selection_Changed(void);
	void Measurement_Selection_Changed(void);
	
	void Update_Measurement_Display(void);
	void Update_Map_Display(void);
	
	void Add_Machine(void);
	void Remove_Machine(void);
	void Edit_Machine(void);
	void Rename_Machine(QWidget *editor);
	
	void Add_Measurement(void);
	void Remove_Measurement(void);
	void Edit_Measurement(void);
	void Rename_Measurement(QWidget *editor);

	void Display_Selected_Item(const int,const int);	// class id, item id
	void Export_Measurements(void);
	void Evaluate_Measurements(void);
	
	void Progress_Cancel(void);
	
	void Help_About(void);
	
protected:
	virtual void closeEvent(QCloseEvent *event);

private:
	
	QAction								*d_add_machine_action;
	QAction								*d_edit_machine_action;
	QAction								*d_remove_machine_action;
	QAction								*d_add_measurement_action;
	QAction								*d_edit_measurement_action;
	QAction								*d_remove_measurement_action;
	QAction								*d_quit_action;
	QListWidget							*d_machines_list;
	QListWidget							*d_measurements_list;
	QToolButton							*d_add_machine_button;
	QToolButton							*d_remove_machine_button;
	QToolButton							*d_edit_machine_button;
	QToolButton							*d_add_measurement_button;
	QToolButton							*d_remove_measurement_button;
	QToolButton							*d_edit_measurement_button;
	
	TDisplayWidget						*d_display_widget;
	TMachineDialog						*d_machine_dialog;
	TMeasurementDialog					*d_measurement_dialog;
	TProgressDialog						*d_progress_dialog;
	TAddDialog							*d_add_dialog;
	TMessageBox							*d_msg_box;

	TMachineComp						d_machine_comp;
	
	bool								d_progress_cancel;
	QString								d_data_path;
	
	QString Active_Machine_Name(void) const;
	QStringList Selected_Measurements(void) const;
	
	QString Pad_String(const QString &s,const int width,const bool add_comma) const;

	void Activate_Machine(const QString &machine_name);
	
	bool Evaluate_Measurement_Expressions(TMachineComp * const machine,std::vector<TMeasurement*> * const measurements, QStringList * const error_list = 0);
	void Update_Measurement_Items(TMachineComp * const machine,std::vector<TMeasurement*> * const measurements);
	QStringList Report_Measurement_Item(TMeasurement* const measurement) const;
	
	bool Get_Min_Max_Ballbar_Error(TMachineComp * const machine,std::vector<TMeasurement*> * const measurements,double * const min,double * const max,double * const min_length,double * const max_length) const;
	bool Get_Min_Max_Laser_Error(TMachineComp * const machine,std::vector<TMeasurement*> * const measurements,double * const min,double * const max,double * const min_length,double * const max_length) const;
	bool Get_Min_Max_ISO_Error(TMachineComp * const machine,std::vector<TMeasurement*> * const measurements,double * const min,double * const max,double * const min_length,double * const max_length) const;
	bool Get_Min_Max_MCG_Error(TMachineComp * const machine,std::vector<TMeasurement*> * const measurements,double * const min,double * const max,double * const min_length,double * const max_length) const;

	void Export_ISO_Measurement(const QString &dir,TMeasurement* const measurement) const;
	void Export_MCG_Measurement(const QString &dir,TMeasurement* const measurement) const;
	void Export_Laser_Measurement(const QString &dir,TMeasurement* const measurement) const;
	void Export_Ballbar_Measurement(const QString &dir,const std::vector<TMeasurement*> &measurements) const;
	
	std::vector<std::vector<TMachineDialog::TMapErrorItem> > Get_Evaluation_Error_Patterns(const std::vector<TMachineDialog::TMapErrorItem> &input_error_items) const;

	bool Update_Machine_Comp(const QString &machine_name, QStringList * const error_list = 0);

	void Update_Machine_Names(void);
	void Update_Measurement_Names(void);

// NOT IMPLEMENTED
	TErrorSimulator(const TErrorSimulator&);
	TErrorSimulator& operator=(const TErrorSimulator&);
};

#endif
