/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef MACHINE_DEFLECTIONCOMPHEADERFILE
#define MACHINE_DEFLECTIONCOMPHEADERFILE

#include <QString>
#include <vector>

#include "../../core/vector3.h"

class TMachineDeflectionComp
{
public:
	// CREATORS
	TMachineDeflectionComp(void);
	~TMachineDeflectionComp(void);
	
	// ACCESSORS
	bool Is_Valid(void) const;
	
	double Evaluate(const double &pos_vert,const double &pos_horz) const;		// calculated from center of horizontal axis
	double Deflection_Sign(void) const {return d_deflection_sign;}
	
	double Axis_Horizontal_Start(void) const {return d_horizontal_axis_start;}
	double Axis_Horizontal_End(void) const {return d_horizontal_axis_end;}
	double Axis_Vertical_Start(void) const {return d_vertical_axis_start;}
	double Axis_Vertical_End(void) const {return d_vertical_axis_end;}
	
	// MANIPULATORS
	void Set_Vertical_Axis(const double &start,const double &end, const double &mech_origin);
	void Set_Horizontal_Axis(const double &start,const double &end, const double &mech_origin);
	
	void Initialize(const double &deflection,const int sign);
	
private:
	
	double								d_vertical_axis_start;
	double								d_vertical_axis_end;
	double								d_vertical_axis_mechanical_origin;
	double								d_horizontal_axis_start;
	double								d_horizontal_axis_end;
	double								d_horizontal_axis_mechanical_origin;
	
	int									d_deflection_sign;
	double								d_deflection_value;
	
	// NOT IMPLEMENTED
	TMachineDeflectionComp(const TMachineDeflectionComp&);
	TMachineDeflectionComp& operator=(const TMachineDeflectionComp&);
};

#endif
