/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef MACHINEPARAMETERWIDGETHEADERFILE
#define MACHINEPARAMETERWIDGETHEADERFILE

#include <QWidget>

class QCheckBox;
class QComboBox;
class QLabel;
class QLineEdit;
class QToolButton;
class TExpressionDialog;

class TMachineParameterWidget : public QWidget
{
Q_OBJECT

public:

// CREATORS
	TMachineParameterWidget(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	~TMachineParameterWidget(void);

// ACCESSORS
	QString Sign_Option(void) const;
	int Sign_Option_Index(void) const;
	bool Is_Option_Enabled(void) const;
	QString Expression(void) const;

// MANIPULATORS
	void Set_Title_Text(const QString &text);
	void Set_Sign_Options(const QStringList &items);
	void Set_Sign_Option_Index(const int index);
	void Set_Option_Enabled(const bool state);
	void Set_Expression(const QString &expression);
	void Set_Min_Max(const double &min,const double &max) {d_min = min;d_max = max;}
	void Enable_Expression_Editor(const bool state);
	void Enable_Expressions(const bool state);

signals:

private slots:
	void Enable_Toggled(bool);
	void Edit_Expression(void);

private:
	QCheckBox							*d_enable_check;
	QComboBox							*d_sign_combo;
	QLabel								*d_title_label;
	QLabel								*d_expression_label;
	QLineEdit							*d_expression_edit;
	QToolButton							*d_edit_button;
	TExpressionDialog					*d_expression_dialog;
	
	double								d_min;
	double								d_max;
	bool								d_enable_expression_editor;
	bool								d_enable_expressions;

// NOT IMPLEMENTED
	TMachineParameterWidget(const TMachineParameterWidget&);
	TMachineParameterWidget& operator=(const TMachineParameterWidget&);
};

#endif
