/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef MEASUREMENTHEADERFILE
#define MEASUREMENTHEADERFILE

#include <QString>
#include <vector>

#include "../../core/vector3.h"

#include "types.h"

class TMeasurement
{
public:
struct TMeasurementItem
{
	TVector3							start_nominal;
	TVector3							end_nominal;
	TVector3							start_actual;
	TVector3							end_actual;
	TVector3							start_map_abc;
	TVector3							end_map_abc;
	TVector3							probe_offset;
	TVector3							axis_nominal;
	TVector3							axis_actual;

	double								length_nominal;
	double								length_actual;
};
	
// CREATORS
	TMeasurement(void);
	virtual ~TMeasurement(void) = 0;
	
// ACCESSORS
	virtual Types::TMeasurementType Type(void) const = 0;
	
	QString Name(void) const {return d_name;}
	int Id(void) const {return d_id;}
	bool Error_State(void) const {return d_error_state;}
	
	int Measurement_Item_Count(void) const {return static_cast<int>(d_measurement_items.size());}
	TMeasurementItem Measurement_Item(const int index) const;
	
	TVector3 Probe_Offset(void) const {return d_probe_offset;}
	
// MANIPULATORS
	
	void Set_Name(const QString &s) {d_name = s;}
	void Set_Error_State(const bool state) {d_error_state = state;}
	virtual void Initialize(void) = 0;	// set default entries
	
	void Clear_Measurement_Items(void) {d_measurement_items.clear();}
	void Add_Measurement_Item(const TMeasurementItem &item) {d_measurement_items.push_back(item);}
	
	void Set_Probe_Offset(const TVector3 &v) {d_probe_offset = v;}
	void Set_Probe_Offset(const double &x,const double &y,const double &z) {d_probe_offset.Set(x,y,z);}
	
private:
	
	int									d_id;
	QString								d_name;

	bool								d_error_state;
	TVector3							d_probe_offset;
	
	std::vector<TMeasurementItem>		d_measurement_items;
	
// NOT IMPLEMENTED
	TMeasurement(const TMeasurement&);
	TMeasurement& operator=(const TMeasurement&);
};

#endif
