/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef MEASUREMENT_BALLBARHEADERFILE
#define MEASUREMENT_BALLBARHEADERFILE

#include <QString>

#include "measurement.h"

class TMeasurement_Ballbar : public TMeasurement
{
public:
// CREATORS
	TMeasurement_Ballbar(void);
	virtual ~TMeasurement_Ballbar(void);
	
// ACCESSORS
	virtual Types::TMeasurementType Type(void) const {return Types::MEASUREMENT_BALLBAR;}
	
	QString Position_X(void) const {return d_position_x;}
	QString Position_Y(void) const {return d_position_y;}
	QString Position_Z(void) const {return d_position_z;}
	
	QString Direction_I(void) const {return d_direction_i;}
	QString Direction_J(void) const {return d_direction_j;}
	QString Direction_K(void) const {return d_direction_k;}
	
	QString Length(void) const {return d_length;}
	
	TVector3 Calc_Position(void) const {return d_calc_position;}
	TVector3 Calc_Direction(void) const {return d_calc_direction;}
	double Calc_Length(void) const {return d_calc_length;}

// MANIPULATORS
	void Set_Position_X(const QString &s) {d_position_x = s;}
	void Set_Position_Y(const QString &s) {d_position_y = s;}
	void Set_Position_Z(const QString &s) {d_position_z = s;}
	
	void Set_Direction_I(const QString &s) {d_direction_i = s;}
	void Set_Direction_J(const QString &s) {d_direction_j = s;}
	void Set_Direction_K(const QString &s) {d_direction_k = s;}
	
	void Set_Length(const QString &s) {d_length = s;}
	
	void Set_Calc_Position(const TVector3 &v) {d_calc_position = v;}
	void Set_Calc_Direction(const TVector3 &v) {d_calc_direction = v;}
	void Set_Calc_Length(const double &d) {d_calc_length = d;}
	
	virtual void Initialize(void);

private:
	QString								d_position_x;
	QString								d_position_y;
	QString								d_position_z;
	QString								d_direction_i;
	QString								d_direction_j;
	QString								d_direction_k;
	QString								d_length;
	
	TVector3							d_calc_position;
	TVector3							d_calc_direction;
	double								d_calc_length;

// NOT IMPLEMENTED
	TMeasurement_Ballbar(const TMeasurement_Ballbar&);
	TMeasurement_Ballbar& operator=(const TMeasurement_Ballbar&);
};


#endif
