/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef MEASUREMENT_ISOHEADERFILE
#define MEASUREMENT_ISOHEADERFILE

#include <QString>

#include "measurement.h"

class TMeasurement_ISO : public TMeasurement
{
public:
// CREATORS
	TMeasurement_ISO(void);
	virtual ~TMeasurement_ISO(void);
	
// ACCESSORS
	virtual Types::TMeasurementType Type(void) const {return Types::MEASUREMENT_ISO;}
	
	QString Start_Position_X(void) const {return d_start_position_x;}
	QString Start_Position_Y(void) const {return d_start_position_y;}
	QString Start_Position_Z(void) const {return d_start_position_z;}
	
	QString End_Position_X(void) const {return d_end_position_x;}
	QString End_Position_Y(void) const {return d_end_position_y;}
	QString End_Position_Z(void) const {return d_end_position_z;}
	
	TVector3 Calc_Start_Position(void) const {return d_calc_start_position;}
	TVector3 Calc_End_Position(void) const {return d_calc_end_position;}

// MANIPULATORS
	void Set_Start_Position_X(const QString &s) {d_start_position_x = s;}
	void Set_Start_Position_Y(const QString &s) {d_start_position_y = s;}
	void Set_Start_Position_Z(const QString &s) {d_start_position_z = s;}
	
	void Set_End_Position_X(const QString &s) {d_end_position_x = s;}
	void Set_End_Position_Y(const QString &s) {d_end_position_y = s;}
	void Set_End_Position_Z(const QString &s) {d_end_position_z = s;}

	void Set_Calc_Start_Position(const TVector3 &v) {d_calc_start_position = v;}
	void Set_Calc_End_Position(const TVector3 &v) {d_calc_end_position = v;}
	
	virtual void Initialize(void);

private:
	
	QString								d_start_position_x;
	QString								d_start_position_y;
	QString								d_start_position_z;
	QString								d_end_position_x;
	QString								d_end_position_y;
	QString								d_end_position_z;
	
	TVector3							d_calc_start_position;
	TVector3							d_calc_end_position;
	
// NOT IMPLEMENTED
	TMeasurement_ISO(const TMeasurement_ISO&);
	TMeasurement_ISO& operator=(const TMeasurement_ISO&);
};


#endif
