/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef MEASUREMENT_MCGHEADERFILE
#define MEASUREMENT_MCGHEADERFILE

#include <QString>

#include "measurement.h"

class TMeasurement_MCG : public TMeasurement
{
public:
// CREATORS
	TMeasurement_MCG(void);
	virtual ~TMeasurement_MCG(void);
	
// ACCESSORS
	virtual Types::TMeasurementType Type(void) const {return Types::MEASUREMENT_MCG;}
	
	QString Position_X(void) const {return d_position_x;}
	QString Position_Y(void) const {return d_position_y;}
	QString Position_Z(void) const {return d_position_z;}
	
	QString Length(void) const {return d_length;}
	QString Azimuth_Count(void) const {return d_azimuth_count;}
	QString Elevation_Count(void) const {return d_elevation_count;}
	QString Start_Elevation(void) const {return d_start_elevation;}
	QString End_Elevation(void) const {return d_end_elevation;}
	
	TVector3 Calc_Position(void) const {return d_calc_position;}
	double Calc_Length(void) const {return d_calc_length;}
	double Calc_Start_Elevation(void) const {return d_calc_start_elevation;}
	double Calc_End_Elevation(void) const {return d_calc_end_elevation;}
	int Calc_Azimuth_Count(void) const {return d_calc_azimuth_count;}
	int Calc_Elevation_Count(void) const {return d_calc_elevation_count;}
	

// MANIPULATORS
	void Set_Position_X(const QString &s) {d_position_x = s;}
	void Set_Position_Y(const QString &s) {d_position_y = s;}
	void Set_Position_Z(const QString &s) {d_position_z = s;}
	
	void Set_Length(const QString &s) {d_length = s;}
	void Set_Azimuth_Count(const QString &s) {d_azimuth_count = s;}
	void Set_Elevation_Count(const QString &s) {d_elevation_count = s;}
	void Set_Start_Elevation(const QString &s) {d_start_elevation = s;}
	void Set_End_Elevation(const QString &s) {d_end_elevation = s;}
	
	void Set_Calc_Position(const TVector3 &v) {d_calc_position = v;}
	void Set_Calc_Length(const double &d) {d_calc_length = d;}
	void Set_Calc_Start_Elevation(const double &d) {d_calc_start_elevation = d;}
	void Set_Calc_End_Elevation(const double &d) {d_calc_end_elevation = d;}
	void Set_Calc_Azimuth_Count(const int &i) {d_calc_azimuth_count = i;}
	void Set_Calc_Elevation_Count(const int &i) {d_calc_elevation_count = i;}

	virtual void Initialize(void);

private:
	QString								d_position_x;
	QString								d_position_y;
	QString								d_position_z;
	QString								d_length;
	QString								d_azimuth_count;
	QString								d_elevation_count;
	QString								d_start_elevation;
	QString								d_end_elevation;
	
	TVector3							d_calc_position;
	double								d_calc_length;
	double								d_calc_start_elevation;
	double								d_calc_end_elevation;
	int									d_calc_azimuth_count;
	int									d_calc_elevation_count;
	
// NOT IMPLEMENTED
	TMeasurement_MCG(const TMeasurement_MCG&);
	TMeasurement_MCG& operator=(const TMeasurement_MCG&);
};


#endif
