/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef MEASUREMENTWIDGETHEADERFILE
#define MEASUREMENTWIDGETHEADERFILE

#include <QDialog>
#include <vector>

#include "measurement.h"
#include "types.h"

class QTabWidget;
class QPushButton;

class TEditor_Laser;
class TEditor_ISO;
class TEditor_MCG;
class TEditor_Ballbar;
class TMeasurement;
class TXmlFile;


class TMeasurementDialog : public QDialog
{
Q_OBJECT

public:

// CREATORS
	TMeasurementDialog(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	~TMeasurementDialog(void);

// ACCESSORS
	QString Last_Error(void) const {return d_last_error;}
	bool Save_Data(void) const;
	
	QStringList Measurement_Names(void) const;
	std::vector<TMeasurement*> Measurements(const QStringList &list);
	
	int Measurement_Id(const QString &name);
	TMeasurement* Measurement(const int id);

// MANIPULATORS
	void Set_Data_Path(const QString &s) {d_data_path = s;}
	bool Load_Data(void);
	
	void Reset(const QString &measurement_name,const QString &measurement_copy_name);
	
	void Remove_Measurement(const int id);
	QString Rename_Measurement(const int id, const QString &new_name);	// returns actual name used.  must be unique

signals:
	void Update_Measurement_Display(void);

private slots:
	void Accept_Clicked(void);

private:
	QTabWidget							*d_measurement_tab;
	QPushButton							*d_accept_button;
	QPushButton							*d_cancel_button;
	TEditor_Laser						*d_editor_laser;
	TEditor_ISO							*d_editor_iso;
	TEditor_MCG							*d_editor_mcg;
	TEditor_Ballbar						*d_editor_ballbar;
	
	QString								d_data_path;
	std::vector<TMeasurement*>			d_measurements;
	int									d_active_measurement_id;
	
	mutable bool						d_data_changed;
	mutable QString						d_last_error;
	
	void Create_Ballbar_Measurement(void);
	void Create_Laser_Measurement(void);
	void Create_ISO_Measurement(void);
	void Create_MCG_Measurement(void);
	
	QString Generate_Unique_Name(const Types::TMeasurementType type,const QString &suggested_name = QString(),const int id = -1) const;
	bool Name_Valid(const QString &name) const;
	
	void Update_Measurement(const int id);
	void Update_Editor(const int id);
	
	bool Save(void) const;
	void Load_Version1(TXmlFile * const xml_file);


// NOT IMPLEMENTED
	TMeasurementDialog(const TMeasurementDialog&);
	TMeasurementDialog& operator=(const TMeasurementDialog&);
};

#endif
