/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QtOpenGL/QGLWidget>
#include <QtOpenGL/QGLPixelBuffer>
#include <QOpenGLShaderProgram>
#include <QPainter>
#include <QRect>
#include <QPoint>
#include <QColor>
#include <assert.h>

#include "../../core/openglshaders.h"

#include "opengldisplaymcg.h"
#include "opengldisplay.h"

static const double						ZERO_EPSILON = 0.0000001;

TOpenGLDisplay::TOpenGLDisplay(
    const QWidget						*parent,
    Qt::WindowFlags 					flags)
:TOpenGLWidget(const_cast<QWidget*>(parent),flags)
{
	d_machine_volume_vertex_id = -1;
	
	// default machine dimensions and coordinate display position
	d_machine_min.Set(-100,-100,-100);
	d_machine_max.Set(100,100,100);
	
	d_axis_draw_pos = (d_machine_max + d_machine_min) / 2.0;
	d_axis_draw_len = (d_machine_max.x - d_machine_min.x) / 5.0;
}

TOpenGLDisplay::~TOpenGLDisplay(void)
{
	this->Clear_Render_Items();
	
	if(d_opengl_vertex_buffer.Has_Buffer(d_machine_volume_vertex_id))
	{
		d_opengl_vertex_buffer.Remove_Buffer(d_machine_volume_vertex_id);
		d_machine_volume_vertex_id = -1;
	}
}

void TOpenGLDisplay::Define_Machine_Volume(
	 const TVector3						&pnt_min,
	 const TVector3						&pnt_max)
{
	TVector3							pt,vec;
	double								*vertex;
	unsigned int						size;
		
	size = 72;	// 12 lines total, 2 verticies, 3 values
	
	if(!d_opengl_vertex_buffer.Has_Buffer(d_machine_volume_vertex_id))
	{
		d_machine_volume_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	}
	
	vertex = d_opengl_vertex_buffer.Buffer(d_machine_volume_vertex_id);
	
	if(vertex)
	{
		vertex[0] = pnt_min.x;
		vertex[1] = pnt_min.y;
		vertex[2] = pnt_min.z;
		
		vertex[3] = pnt_max.x;
		vertex[4] = pnt_min.y;
		vertex[5] = pnt_min.z;
		
		
		vertex[6] = pnt_max.x;
		vertex[7] = pnt_min.y;
		vertex[8] = pnt_min.z;

		vertex[9] = pnt_max.x;
		vertex[10] = pnt_max.y;
		vertex[11] = pnt_min.z;

		
		vertex[12] = pnt_max.x;
		vertex[13] = pnt_max.y;
		vertex[14] = pnt_min.z;
		
		vertex[15] = pnt_min.x;
		vertex[16] = pnt_max.y;
		vertex[17] = pnt_min.z;

		
		vertex[18] = pnt_min.x;
		vertex[19] = pnt_max.y;
		vertex[20] = pnt_min.z;
		
		vertex[21] = pnt_min.x;
		vertex[22] = pnt_min.y;
		vertex[23] = pnt_min.z;

		
		vertex[24] = pnt_min.x;
		vertex[25] = pnt_min.y;
		vertex[26] = pnt_max.z;
		
		vertex[27] = pnt_max.x;
		vertex[28] = pnt_min.y;
		vertex[29] = pnt_max.z;
		
		
		vertex[30] = pnt_max.x;
		vertex[31] = pnt_min.y;
		vertex[32] = pnt_max.z;

		vertex[33] = pnt_max.x;
		vertex[34] = pnt_max.y;
		vertex[35] = pnt_max.z;

		
		vertex[36] = pnt_max.x;
		vertex[37] = pnt_max.y;
		vertex[38] = pnt_max.z;

		vertex[39] = pnt_min.x;
		vertex[40] = pnt_max.y;
		vertex[41] = pnt_max.z;
		
		
		vertex[42] = pnt_min.x;
		vertex[43] = pnt_max.y;
		vertex[44] = pnt_max.z;
		
		vertex[45] = pnt_min.x;
		vertex[46] = pnt_min.y;
		vertex[47] = pnt_max.z;

		
		vertex[48] = pnt_min.x;
		vertex[49] = pnt_min.y;
		vertex[50] = pnt_min.z;
		
		vertex[51] = pnt_min.x;
		vertex[52] = pnt_min.y;
		vertex[53] = pnt_max.z;
		
		
		vertex[54] = pnt_max.x;
		vertex[55] = pnt_min.y;
		vertex[56] = pnt_min.z;
		
		vertex[57] = pnt_max.x;
		vertex[58] = pnt_min.y;
		vertex[59] = pnt_max.z;
		
		
		vertex[60] = pnt_max.x;
		vertex[61] = pnt_max.y;
		vertex[62] = pnt_min.z;
		
		vertex[63] = pnt_max.x;
		vertex[64] = pnt_max.y;
		vertex[65] = pnt_max.z;
		
		
		vertex[66] = pnt_min.x;
		vertex[67] = pnt_max.y;
		vertex[68] = pnt_min.z;
		
		vertex[69] = pnt_min.x;
		vertex[70] = pnt_max.y;
		vertex[71] = pnt_max.z;
	}
	
	d_axis_draw_pos = pnt_min;
	
	d_machine_min = pnt_min;
	d_machine_max = pnt_max;
	
	pt = d_machine_max - d_machine_min;
	vec = pt;
	
	// find shortest vector
	d_axis_draw_len = pt.x;
	
	if(pt.y < d_axis_draw_len)
	{
		d_axis_draw_len = pt.y;
	}
	
	if(pt.z < d_axis_draw_len)
	{
		d_axis_draw_len = pt.z;
	}
	
	if(vec.Length() > ZERO_EPSILON)
	{
		vec.Normal();
		
		d_axis_draw_len /= 5;

		d_axis_draw_pos = d_machine_min - vec * d_axis_draw_len;
	}
}

void TOpenGLDisplay::Clear_Render_Items(void)
{
	std::vector<TRenderBallbarItem>::iterator ballbar_iter;
	std::vector<TRenderMCGItem>::iterator mcg_iter;
	std::vector<TRenderISOItem>::iterator iso_iter;
	std::vector<TRenderLaserItem>::iterator laser_iter;
	
	this->Clear_Selection_Items();
	
	for(ballbar_iter = d_ballbar_items.begin();ballbar_iter != d_ballbar_items.end();++ballbar_iter)
	{
		d_opengl_vertex_buffer.Remove_Buffer((*ballbar_iter).ballbar_shaft_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*ballbar_iter).ballbar_shaft_normal_id);
		
		d_opengl_vertex_buffer.Remove_Buffer((*ballbar_iter).ballbar_sphere_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*ballbar_iter).ballbar_sphere_normal_id);
		
		(*ballbar_iter).ballbar_shaft_vertex_id = -1;
		(*ballbar_iter).ballbar_shaft_normal_id = -1;
		
		(*ballbar_iter).ballbar_sphere_vertex_id = -1;
		(*ballbar_iter).ballbar_sphere_normal_id = -1;
	}
	
	d_ballbar_items.clear();
	
	for(mcg_iter = d_mcg_base_items.begin();mcg_iter != d_mcg_base_items.end();++mcg_iter)
	{
		d_opengl_vertex_buffer.Remove_Buffer((*mcg_iter).mcg_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*mcg_iter).mcg_nominal_id);

		(*mcg_iter).mcg_vertex_id = -1;
		(*mcg_iter).mcg_nominal_id = -1;
	}
	
	d_mcg_base_items.clear();
		
	for(mcg_iter = d_mcg_arm_items.begin();mcg_iter != d_mcg_arm_items.end();++mcg_iter)
	{
		d_opengl_vertex_buffer.Remove_Buffer((*mcg_iter).mcg_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*mcg_iter).mcg_nominal_id);

		(*mcg_iter).mcg_vertex_id = -1;
		(*mcg_iter).mcg_nominal_id = -1;
	}
	
	d_mcg_arm_items.clear();

	for(iso_iter = d_isogauge_items.begin();iso_iter != d_isogauge_items.end();++iso_iter)
	{
		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face1_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face1_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face2_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face2_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face3_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face3_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face4_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face4_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face5_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face5_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face6_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).face6_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).probe_stem_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).probe_stem_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).probe_ruby_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*iso_iter).probe_ruby_normal_id);

		(*iso_iter).face1_vertex_id = -1;
		(*iso_iter).face1_normal_id = -1;
		(*iso_iter).face2_vertex_id = -1;
		(*iso_iter).face2_normal_id = -1;
		(*iso_iter).face3_vertex_id = -1;
		(*iso_iter).face3_normal_id = -1;
		(*iso_iter).face4_vertex_id = -1;
		(*iso_iter).face4_normal_id = -1;
		(*iso_iter).face5_vertex_id = -1;
		(*iso_iter).face5_normal_id = -1;
		(*iso_iter).face6_vertex_id = -1;
		(*iso_iter).face6_normal_id = -1;
		(*iso_iter).probe_stem_vertex_id = -1;
		(*iso_iter).probe_stem_normal_id = -1;
		(*iso_iter).probe_ruby_vertex_id = -1;
		(*iso_iter).probe_ruby_normal_id = -1;
	}

	d_isogauge_items.clear();
	
	for(laser_iter = d_laser_items.begin();laser_iter != d_laser_items.end();++laser_iter)
	{
		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).cube1_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).cube1_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).shaft_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).shaft_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).cube2_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).cube2_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).probe_z_stem_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).probe_z_stem_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).probe_joint_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).probe_joint_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).probe_xy_stem_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).probe_xy_stem_normal_id);

		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).probe_ruby_vertex_id);
		d_opengl_vertex_buffer.Remove_Buffer((*laser_iter).probe_ruby_normal_id);

		(*laser_iter).cube1_vertex_id = -1;
		(*laser_iter).cube1_normal_id = -1;
		(*laser_iter).shaft_vertex_id = -1;
		(*laser_iter).shaft_normal_id = -1;
		(*laser_iter).cube2_vertex_id = -1;
		(*laser_iter).cube2_normal_id = -1;
		(*laser_iter).probe_z_stem_vertex_id = -1;
		(*laser_iter).probe_z_stem_normal_id = -1;
		(*laser_iter).probe_joint_vertex_id = -1;
		(*laser_iter).probe_joint_normal_id = -1;
		(*laser_iter).probe_xy_stem_vertex_id = -1;
		(*laser_iter).probe_xy_stem_normal_id = -1;
		(*laser_iter).probe_ruby_vertex_id = -1;
		(*laser_iter).probe_ruby_normal_id = -1;
	}

	d_laser_items.clear();
}

void TOpenGLDisplay::Clear_Render_Volume(void)
{
	if(d_opengl_vertex_buffer.Has_Buffer(d_machine_volume_vertex_id))
	{
		d_opengl_vertex_buffer.Remove_Buffer(d_machine_volume_vertex_id);
		d_machine_volume_vertex_id = -1;
	}
}

void TOpenGLDisplay::Add_BallBar(
	const TBallBar						&ballbar)
{
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex>::const_iterator iter;
	TRenderBallbarItem					ballbar_item;
	TSelectionItemType					si;
	double								*vertex;
	double								*normal;
	unsigned int						size;
	unsigned int						ucntr;
	
	ballbar_item.id = ballbar.id;
	ballbar_item.name = ballbar.name;
	ballbar_item.selected = ballbar.selected;
	ballbar_item.error = ballbar.error;
	ballbar_item.label_coordinates = ballbar.label_coordinates;
	
	size = ballbar.ballbar_shaft.size() * 3;
	
	ballbar_item.ballbar_shaft_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	ballbar_item.ballbar_shaft_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(ballbar_item.ballbar_shaft_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(ballbar_item.ballbar_shaft_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = ballbar.ballbar_shaft.begin();iter != ballbar.ballbar_shaft.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = ballbar.ballbar_spheres.size() * 3;
	
	ballbar_item.ballbar_sphere_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	ballbar_item.ballbar_sphere_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(ballbar_item.ballbar_sphere_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(ballbar_item.ballbar_sphere_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = ballbar.ballbar_spheres.begin();iter != ballbar.ballbar_spheres.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = ballbar.probe_stem.size() * 3;
	
	ballbar_item.probe_stem_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	ballbar_item.probe_stem_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(ballbar_item.probe_stem_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(ballbar_item.probe_stem_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = ballbar.probe_stem.begin();iter != ballbar.probe_stem.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = ballbar.probe_ruby.size() * 3;
	
	ballbar_item.probe_ruby_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	ballbar_item.probe_ruby_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(ballbar_item.probe_ruby_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(ballbar_item.probe_ruby_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = ballbar.probe_ruby.begin();iter != ballbar.probe_ruby.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	d_ballbar_items.push_back(ballbar_item);
	
	si.selection_id = ballbar.id;
	si.min_corner = ballbar.min_xyz;
	si.max_corner = ballbar.max_xyz;
	si.selection_mode = TOpenGLWidget::SELECTION_LINE;
	
	this->AddSelectionItem(si);
}

void TOpenGLDisplay::Add_MCGGauge(
	const TMCGGauge						&mcg)
{
	std::vector<MCG::TMCGData>			mcg_data;
	std::vector<MCG::TMCGData>::iterator iter;
	std::vector<TVector3>::iterator		vertex_iter,normal_iter;
	TRenderMCGItem						mcg_base_item;
	TRenderMCGItem						mcg_arm_item;
	TSelectionItemType					si;
	double								*vertex;
	double								*normal;
	unsigned int						size;
	unsigned int						ucntr;

	if(!d_mcg_base_items.size())
	{
		mcg_data = MCG::Render_MCG_Base(mcg.center);
		
		mcg_base_item.id = -1;
		mcg_base_item.selected = false;
		mcg_base_item.error = mcg.error;
		
		for(iter = mcg_data.begin();iter != mcg_data.end();++iter)
		{
			size = (*iter).verticies.size() * 3;
						
			mcg_base_item.mode = (*iter).mode;
			mcg_base_item.color = (*iter).color;
			
			mcg_base_item.mcg_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
			mcg_base_item.mcg_nominal_id = d_opengl_vertex_buffer.Create_Buffer(size);
			
			vertex = d_opengl_vertex_buffer.Buffer(mcg_base_item.mcg_vertex_id);
			normal = d_opengl_vertex_buffer.Buffer(mcg_base_item.mcg_nominal_id);
			
			if(vertex && normal)
			{
				ucntr = 0;
				for(vertex_iter = (*iter).verticies.begin(),normal_iter = (*iter).normals.begin();vertex_iter != (*iter).verticies.end();++vertex_iter,++normal_iter)
				{
					vertex[ucntr] = (*vertex_iter).x;
					vertex[ucntr+1] = (*vertex_iter).y;
					vertex[ucntr+2] = (*vertex_iter).z;
					
					normal[ucntr] = (*normal_iter).i;
					normal[ucntr+1] = (*normal_iter).j;
					normal[ucntr+2] = (*normal_iter).k;
					
					ucntr += 3;
				}
			}
			
			d_mcg_base_items.push_back(mcg_base_item);
		}
	}
	
	mcg_data = MCG::Render_MCG_Arm(mcg.center,mcg.axis,mcg.length);
	
	mcg_arm_item.id = mcg.id;
	mcg_arm_item.selected = mcg.selected;
					
	for(iter = mcg_data.begin();iter != mcg_data.end();++iter)
	{
		size = (*iter).verticies.size() * 3;
					
		mcg_arm_item.mode = (*iter).mode;
		mcg_arm_item.color = (*iter).color;
				
		mcg_arm_item.mcg_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
		mcg_arm_item.mcg_nominal_id = d_opengl_vertex_buffer.Create_Buffer(size);
		
		vertex = d_opengl_vertex_buffer.Buffer(mcg_arm_item.mcg_vertex_id);
		normal = d_opengl_vertex_buffer.Buffer(mcg_arm_item.mcg_nominal_id);
		
		if(vertex && normal)
		{
			ucntr = 0;
			for(vertex_iter = (*iter).verticies.begin(),normal_iter = (*iter).normals.begin();vertex_iter != (*iter).verticies.end();++vertex_iter,++normal_iter)
			{
				vertex[ucntr] = (*vertex_iter).x;
				vertex[ucntr+1] = (*vertex_iter).y;
				vertex[ucntr+2] = (*vertex_iter).z;
				
				normal[ucntr] = (*normal_iter).i;
				normal[ucntr+1] = (*normal_iter).j;
				normal[ucntr+2] = (*normal_iter).k;
				
				ucntr += 3;
			}
		}
		
		d_mcg_arm_items.push_back(mcg_arm_item);
	}
	
	si.selection_id = mcg.id;
	si.min_corner = mcg.min_xyz;
	si.max_corner = mcg.max_xyz;
	si.selection_mode = TOpenGLWidget::SELECTION_LINE;

	this->AddSelectionItem(si);
}

void TOpenGLDisplay::Add_ISOGauge(
	const TISOGauge						&iso)
{
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex>::const_iterator iter;
	TRenderISOItem						iso_item;
	TSelectionItemType					si;
	double								*vertex;
	double								*normal;
	unsigned int						size;
	unsigned int						ucntr;
	
	iso_item.id = iso.id;
	iso_item.selected = iso.selected;
	iso_item.error = iso.error;

	size = iso.face1.size() * 3;
	
	iso_item.face1_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	iso_item.face1_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(iso_item.face1_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(iso_item.face1_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = iso.face1.begin();iter != iso.face1.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = iso.face2.size() * 3;
	
	iso_item.face2_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	iso_item.face2_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(iso_item.face2_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(iso_item.face2_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = iso.face2.begin();iter != iso.face2.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = iso.face3.size() * 3;
	
	iso_item.face3_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	iso_item.face3_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(iso_item.face3_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(iso_item.face3_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = iso.face3.begin();iter != iso.face3.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = iso.face4.size() * 3;
	
	iso_item.face4_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	iso_item.face4_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(iso_item.face4_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(iso_item.face4_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = iso.face4.begin();iter != iso.face4.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = iso.face5.size() * 3;
	
	iso_item.face5_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	iso_item.face5_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(iso_item.face5_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(iso_item.face5_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = iso.face5.begin();iter != iso.face5.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = iso.face6.size() * 3;
	
	iso_item.face6_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	iso_item.face6_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(iso_item.face6_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(iso_item.face6_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = iso.face6.begin();iter != iso.face6.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = iso.probe_stem.size() * 3;
	
	iso_item.probe_stem_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	iso_item.probe_stem_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(iso_item.probe_stem_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(iso_item.probe_stem_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = iso.probe_stem.begin();iter != iso.probe_stem.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
		
	size = iso.probe_ruby.size() * 3;
	
	iso_item.probe_ruby_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	iso_item.probe_ruby_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(iso_item.probe_ruby_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(iso_item.probe_ruby_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = iso.probe_ruby.begin();iter != iso.probe_ruby.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}

	d_isogauge_items.push_back(iso_item);
	
	si.selection_id = iso.id;
	si.min_corner = iso.min_xyz;
	si.max_corner = iso.max_xyz;
	
	si.axis_1 = iso.axis1;
	si.axis_2 = iso.axis2;
	si.axis_3 = iso.axis3;
	
	si.selection_mode = TOpenGLWidget::SELECTION_BOX;

	this->AddSelectionItem(si);
}

void TOpenGLDisplay::Add_Laser(
	const TLaser						&laser)
{
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex>::const_iterator iter;
	TRenderLaserItem					laser_item;
	TSelectionItemType					si;
	double								*vertex;
	double								*normal;
	unsigned int						size;
	unsigned int						ucntr;
	
	laser_item.id = laser.id;
	laser_item.selected = laser.selected;
	laser_item.error = laser.error;
	
	size = laser.cube1.size() * 3;
	
	laser_item.cube1_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	laser_item.cube1_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(laser_item.cube1_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(laser_item.cube1_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = laser.cube1.begin();iter != laser.cube1.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = laser.shaft.size() * 3;
	
	laser_item.shaft_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	laser_item.shaft_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(laser_item.shaft_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(laser_item.shaft_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = laser.shaft.begin();iter != laser.shaft.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
	
	size = laser.cube2.size() * 3;
	
	laser_item.cube2_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	laser_item.cube2_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(laser_item.cube2_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(laser_item.cube2_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = laser.cube2.begin();iter != laser.cube2.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
		
	size = laser.probe_z_stem.size() * 3;
	
	laser_item.probe_z_stem_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	laser_item.probe_z_stem_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(laser_item.probe_z_stem_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(laser_item.probe_z_stem_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = laser.probe_z_stem.begin();iter != laser.probe_z_stem.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
		
	size = laser.probe_joint.size() * 3;
	
	laser_item.probe_joint_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	laser_item.probe_joint_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(laser_item.probe_joint_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(laser_item.probe_joint_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = laser.probe_joint.begin();iter != laser.probe_joint.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
		
	size = laser.probe_xy_stem.size() * 3;
	
	laser_item.probe_xy_stem_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	laser_item.probe_xy_stem_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(laser_item.probe_xy_stem_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(laser_item.probe_xy_stem_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = laser.probe_xy_stem.begin();iter != laser.probe_xy_stem.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}
		
	size = laser.probe_ruby.size() * 3;
	
	laser_item.probe_ruby_vertex_id = d_opengl_vertex_buffer.Create_Buffer(size);
	laser_item.probe_ruby_normal_id = d_opengl_vertex_buffer.Create_Buffer(size);
	
	vertex = d_opengl_vertex_buffer.Buffer(laser_item.probe_ruby_vertex_id);
	normal = d_opengl_vertex_buffer.Buffer(laser_item.probe_ruby_normal_id);
	
	if(vertex && normal)
	{
		ucntr = 0;
		for(iter = laser.probe_ruby.begin();iter != laser.probe_ruby.end();++iter)
		{
			vertex[ucntr] = (*iter).vertex.x;
			vertex[ucntr+1] = (*iter).vertex.y;
			vertex[ucntr+2] = (*iter).vertex.z;
			
			normal[ucntr] = (*iter).normal.i;
			normal[ucntr+1] = (*iter).normal.j;
			normal[ucntr+2] = (*iter).normal.k;
			
			ucntr += 3;
		}
	}

	d_laser_items.push_back(laser_item);
	
	si.selection_id = laser.id;
	si.min_corner = laser.min_xyz;
	si.max_corner = laser.max_xyz;
	
	si.axis_1 = laser.axis1;
	si.axis_2 = laser.axis2;
	si.axis_3 = laser.axis3;
	
	si.selection_mode = TOpenGLWidget::SELECTION_BOX;

	this->AddSelectionItem(si);
}

void TOpenGLDisplay::Clear_Selection(void)
{
	std::vector<TOpenGLDisplay::TRenderBallbarItem>::iterator ballbar_iter;
	std::vector<TOpenGLDisplay::TRenderMCGItem>::iterator mcg_iter;
	std::vector<TOpenGLDisplay::TRenderISOItem>::iterator iso_iter;
	std::vector<TOpenGLDisplay::TRenderLaserItem>::iterator laser_iter;
	
	for(ballbar_iter = d_ballbar_items.begin();ballbar_iter != d_ballbar_items.end();++ballbar_iter)
	{
		(*ballbar_iter).selected = false;
	}
	
	for(mcg_iter = d_mcg_arm_items.begin();mcg_iter != d_mcg_arm_items.end();++mcg_iter)
	{
		(*mcg_iter).selected = false;
	}
	
	for(iso_iter = d_isogauge_items.begin();iso_iter != d_isogauge_items.end();++iso_iter)
	{
		(*iso_iter).selected = false;
	}
	
	for(laser_iter = d_laser_items.begin();laser_iter != d_laser_items.end();++laser_iter)
	{
		(*laser_iter).selected = false;
	}
	
	this->update();
}

void TOpenGLDisplay::Set_Selection(
	const int							id)
{
	std::vector<TOpenGLDisplay::TRenderBallbarItem>::iterator ballbar_iter;
	std::vector<TOpenGLDisplay::TRenderMCGItem>::iterator mcg_iter;
	std::vector<TOpenGLDisplay::TRenderISOItem>::iterator iso_iter;
	std::vector<TOpenGLDisplay::TRenderLaserItem>::iterator laser_iter;

	this->Clear_Selection();

	for(ballbar_iter = d_ballbar_items.begin();ballbar_iter != d_ballbar_items.end();++ballbar_iter)
	{
		if((*ballbar_iter).id == id)
		{
			(*ballbar_iter).selected = true;
			break;
		}
	}

	for(mcg_iter = d_mcg_arm_items.begin();mcg_iter != d_mcg_arm_items.end();++mcg_iter)
	{
		if((*mcg_iter).id == id)
		{
			(*mcg_iter).selected = true;
		}
	}
	
	for(iso_iter = d_isogauge_items.begin();iso_iter != d_isogauge_items.end();++iso_iter)
	{
		if((*iso_iter).id == id)
		{
			(*iso_iter).selected = true;
			break;
		}
	}
	
	for(laser_iter = d_laser_items.begin();laser_iter != d_laser_items.end();++laser_iter)
	{
		if((*laser_iter).id == id)
		{
			(*laser_iter).selected = true;
			break;
		}
	}
	
	this->update();
}

void TOpenGLDisplay::GLInit(void)
{
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glFrontFace(GL_CCW);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	glShadeModel(GL_SMOOTH);
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	
	d_triangle_shader_program = new QOpenGLShaderProgram(this);
	d_triangle_shader_program->addShaderFromSourceCode(QOpenGLShader::Vertex,OpenGLShaders::triangle_vertex_shader);
	d_triangle_shader_program->addShaderFromSourceCode(QOpenGLShader::Fragment,OpenGLShaders::triangle_fragment_shader);
	d_triangle_shader_program->link();
	
	d_triangle_uniform_color = d_triangle_shader_program->uniformLocation("color");
	d_triangle_uniform_specular = d_triangle_shader_program->uniformLocation("specular");
	d_triangle_uniform_offset = d_triangle_shader_program->uniformLocation("offset");
	d_triangle_attribute_vertex = d_triangle_shader_program->attributeLocation("vertex");
	d_triangle_attribute_normal = d_triangle_shader_program->attributeLocation("normal");
	
	d_basic_shader_program = new QOpenGLShaderProgram(this);
	d_basic_shader_program->addShaderFromSourceCode(QOpenGLShader::Vertex,OpenGLShaders::basic_vertex_shader);
	d_basic_shader_program->addShaderFromSourceCode(QOpenGLShader::Fragment,OpenGLShaders::basic_fragment_shader);
	d_basic_shader_program->link();
	
	d_basic_uniform_color = d_basic_shader_program->uniformLocation("color");
	d_basic_attribute_vertex = d_basic_shader_program->attributeLocation("vertex");
	
	glClearColor(1.0f,1.0f,1.0f,1.0f);
	
	// default view
	this->Rotate_Model(TVector3(1,0,0),-1.31);	// 75 degrees
	this->Rotate_Model(TVector3(0,0,1),-0.26);	// 15 degrees
}

void TOpenGLDisplay::GLResize(const int X,const int Y)
{
	Q_UNUSED(X);
	Q_UNUSED(Y);
}

void TOpenGLDisplay::GLRender(void)
{
	std::vector<TRenderBallbarItem>::const_iterator ballbar_iter;
	std::vector<TRenderMCGItem>::const_iterator mcg_iter;
	std::vector<TRenderISOItem>::const_iterator iso_iter;
	std::vector<TRenderLaserItem>::const_iterator laser_iter;
	TVector3							pt1,pt2;
	TVector3							axis_display_positions[3];
	TVector3							color;
	QString								text;
	QRect								rect;
	QColor								label_color(224,224,224,160);
	int									text_x,text_y;
	double								specular;
	double								opacity;
	unsigned int						ucntr;
	unsigned int						ucount;
	double								axis_lines[18];				// 3 lines, 2 points, 3 axis
	double								*vertex;
	
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	
	// X Axis display line
	
	pt2 = d_axis_draw_pos + TVector3(1,0,0) * d_axis_draw_len;
	axis_display_positions[0] = pt2 + TVector3(1,0,0) * d_axis_draw_len/8;
	axis_lines[0] = d_axis_draw_pos.x;
	axis_lines[1] = d_axis_draw_pos.y;
	axis_lines[2] = d_axis_draw_pos.z;
	axis_lines[3] = pt2.x;
	axis_lines[4] = pt2.y;
	axis_lines[5] = pt2.z;
	
	// Y Axis display line
	
	pt2 = d_axis_draw_pos + TVector3(0,1,0) * d_axis_draw_len;
	axis_display_positions[1] = pt2 + TVector3(0,1,0) * d_axis_draw_len/8;
	axis_lines[6] = d_axis_draw_pos.x;
	axis_lines[7] = d_axis_draw_pos.y;
	axis_lines[8] = d_axis_draw_pos.z;
	axis_lines[9] = pt2.x;
	axis_lines[10] = pt2.y;
	axis_lines[11] = pt2.z;
	
	// Z Axis display line
	
	pt2 = d_axis_draw_pos + TVector3(0,0,1) * d_axis_draw_len;
	axis_display_positions[2] = pt2 + TVector3(0,0,1) * d_axis_draw_len/8;
	axis_lines[12] = d_axis_draw_pos.x;
	axis_lines[13] = d_axis_draw_pos.y;
	axis_lines[14] = d_axis_draw_pos.z;
	axis_lines[15] = pt2.x;
	axis_lines[16] = pt2.y;
	axis_lines[17] = pt2.z;
	
	if(this->Is_Processing_Scale_To_Fit())
	{
		if(d_opengl_vertex_buffer.Has_Buffer(d_machine_volume_vertex_id))
		{
			this->Process_Vertex(d_machine_min);
			this->Process_Vertex(d_machine_max);
			
			this->Process_Vertex(d_machine_max.x,d_machine_min.y,d_machine_min.z);
			this->Process_Vertex(d_machine_max.x,d_machine_max.y,d_machine_min.z);
			this->Process_Vertex(d_machine_min.x,d_machine_max.y,d_machine_min.z);
			
			this->Process_Vertex(d_machine_max.x,d_machine_min.y,d_machine_max.z);
			this->Process_Vertex(d_machine_max.x,d_machine_max.y,d_machine_max.z);
			this->Process_Vertex(d_machine_min.x,d_machine_max.y,d_machine_max.z);
		}
		
		for(mcg_iter = d_mcg_base_items.begin();mcg_iter != d_mcg_base_items.end();++mcg_iter)
		{
			if(d_opengl_vertex_buffer.Has_Buffer((*mcg_iter).mcg_vertex_id))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*mcg_iter).mcg_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*mcg_iter).mcg_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
		}
				
		for(mcg_iter = d_mcg_arm_items.begin();mcg_iter != d_mcg_arm_items.end();++mcg_iter)
		{
			if(d_opengl_vertex_buffer.Has_Buffer((*mcg_iter).mcg_vertex_id))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*mcg_iter).mcg_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*mcg_iter).mcg_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
		}
		
		if(d_ballbar_items.size() == 0)
		{
			this->Process_Vertex(d_axis_draw_pos - TVector3(d_axis_draw_len,d_axis_draw_len,d_axis_draw_len));
			
			this->Process_Vertex(axis_lines[3] + d_axis_draw_len,axis_lines[4],axis_lines[5]);
			this->Process_Vertex(axis_lines[9],axis_lines[10] + d_axis_draw_len,axis_lines[11]);
			this->Process_Vertex(axis_lines[15],axis_lines[16],axis_lines[17] + d_axis_draw_len);
		}
		else
		{
			this->Process_Vertex(d_axis_draw_pos);
			
			this->Process_Vertex(axis_lines[3],axis_lines[4],axis_lines[5]);
			this->Process_Vertex(axis_lines[9],axis_lines[10],axis_lines[11]);
			this->Process_Vertex(axis_lines[15],axis_lines[16],axis_lines[17]);
		}
		
		for(ballbar_iter = d_ballbar_items.begin();ballbar_iter != d_ballbar_items.end();++ballbar_iter)
		{
			if(!((*ballbar_iter).ballbar_shaft_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*ballbar_iter).ballbar_shaft_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*ballbar_iter).ballbar_shaft_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*ballbar_iter).ballbar_sphere_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*ballbar_iter).ballbar_sphere_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*ballbar_iter).ballbar_sphere_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*ballbar_iter).probe_stem_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*ballbar_iter).probe_stem_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*ballbar_iter).probe_stem_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
		}
		
		for(iso_iter = d_isogauge_items.begin();iso_iter != d_isogauge_items.end();++iso_iter)
		{
			if(!((*iso_iter).face1_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*iso_iter).face1_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face1_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*iso_iter).face2_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*iso_iter).face2_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face2_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*iso_iter).face3_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*iso_iter).face3_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face3_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*iso_iter).face4_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*iso_iter).face4_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face4_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*iso_iter).face5_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*iso_iter).face5_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face5_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*iso_iter).face6_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*iso_iter).face6_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face6_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*iso_iter).probe_stem_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*iso_iter).probe_stem_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*iso_iter).probe_stem_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
		}
		
		for(laser_iter = d_laser_items.begin();laser_iter != d_laser_items.end();++laser_iter)
		{
			if(!((*laser_iter).cube1_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*laser_iter).cube1_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*laser_iter).cube1_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*laser_iter).shaft_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*laser_iter).shaft_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*laser_iter).shaft_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*laser_iter).cube2_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*laser_iter).cube2_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*laser_iter).cube2_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*laser_iter).probe_z_stem_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*laser_iter).probe_z_stem_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*laser_iter).probe_z_stem_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			if(!((*laser_iter).probe_joint_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*laser_iter).probe_joint_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*laser_iter).probe_joint_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
			
			if(!((*laser_iter).probe_xy_stem_vertex_id < 0))
			{
				vertex = d_opengl_vertex_buffer.Buffer((*laser_iter).probe_xy_stem_vertex_id);
				ucount = d_opengl_vertex_buffer.Buffer_Size((*laser_iter).probe_xy_stem_vertex_id);
				
				if(vertex)
				{
					for(ucntr = 0;ucntr < ucount;ucntr += 3)
					{
						this->Process_Vertex(vertex[ucntr],vertex[ucntr+1],vertex[ucntr+2]);
					}
				}
			}
		}
	}
	else
	{
		d_triangle_shader_program->bind();
		
		opacity = 1.0;
		specular = 0.1;
		
		for(mcg_iter = d_mcg_base_items.begin();mcg_iter != d_mcg_base_items.end();++mcg_iter)
		{
			if(d_opengl_vertex_buffer.Has_Buffer((*mcg_iter).mcg_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*mcg_iter).mcg_nominal_id))
			{
				color = (*mcg_iter).color;
				
				if((*mcg_iter).error)
				{
					color.x += 0.75;
					if(color.x > 1.0) color.x = 1.0;
				}

				this->Render_Triangle_Buffer((*mcg_iter).mode,
											 d_opengl_vertex_buffer.Buffer_Size((*mcg_iter).mcg_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*mcg_iter).mcg_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*mcg_iter).mcg_nominal_id));
			}
		}

		for(mcg_iter = d_mcg_arm_items.begin();mcg_iter != d_mcg_arm_items.end();++mcg_iter)
		{
			if(d_opengl_vertex_buffer.Has_Buffer((*mcg_iter).mcg_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*mcg_iter).mcg_nominal_id))
			{
				color = (*mcg_iter).color;
				
				if((*mcg_iter).selected)
				{
					color += TVector3(0,0,0.75);
					if(color.z > 1.0) color.z = 1.0;
				}
				
				if((*mcg_iter).error)
				{
					color.x += 0.75;
					if(color.x > 1.0) color.x = 1.0;
				}

				this->Render_Triangle_Buffer((*mcg_iter).mode,
											 d_opengl_vertex_buffer.Buffer_Size((*mcg_iter).mcg_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*mcg_iter).mcg_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*mcg_iter).mcg_nominal_id));
			}
		}
		
		for(ballbar_iter = d_ballbar_items.begin();ballbar_iter != d_ballbar_items.end();++ballbar_iter)
		{
			if(d_opengl_vertex_buffer.Has_Buffer((*ballbar_iter).ballbar_shaft_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*ballbar_iter).ballbar_shaft_normal_id))
			{
				color.Set(0.1,0.1,0.1);
				specular = 0.1;

				if((*ballbar_iter).selected)
				{
					color += TVector3(0,0,0.75);
					if(color.z > 1.0) color.z = 1.0;
				}
				
				if((*ballbar_iter).error)
				{
					color.x += 0.75;
					if(color.x > 1.0) color.x = 1.0;
				}
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*ballbar_iter).ballbar_shaft_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*ballbar_iter).ballbar_shaft_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*ballbar_iter).ballbar_shaft_normal_id));
			}
			
			if(d_opengl_vertex_buffer.Has_Buffer((*ballbar_iter).ballbar_sphere_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*ballbar_iter).ballbar_sphere_normal_id))
			{				
				color.Set(0.55,0.55,0.6);
				specular = 0.4;

				if((*ballbar_iter).selected)
				{
					color += TVector3(-0.2,-0.2,0.75);
					if(color.x < 0.0) color.x = 0.0;
					if(color.y < 0.0) color.y = 0.0;
					if(color.z > 1.0) color.z = 1.0;
				}
				
				if((*ballbar_iter).error)
				{
					color.x += 0.75;
					if(color.x > 1.0) color.x = 1.0;
				}
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*ballbar_iter).ballbar_sphere_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*ballbar_iter).ballbar_sphere_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*ballbar_iter).ballbar_sphere_normal_id));
				
			}
			
			if(d_opengl_vertex_buffer.Has_Buffer((*ballbar_iter).probe_stem_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*ballbar_iter).probe_stem_normal_id))
			{
				color.Set(0.7,0.7,0.7);
				specular = 0.1;
			
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*ballbar_iter).probe_stem_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*ballbar_iter).probe_stem_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*ballbar_iter).probe_stem_normal_id));
				
			}
			
			if(d_opengl_vertex_buffer.Has_Buffer((*ballbar_iter).probe_ruby_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*ballbar_iter).probe_ruby_normal_id))
			{
				color.Set(1.0,0.0,0.0);
				specular = 0.1;
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*ballbar_iter).probe_ruby_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*ballbar_iter).probe_ruby_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*ballbar_iter).probe_ruby_normal_id));
				
			}
		}
		
		for(iso_iter = d_isogauge_items.begin();iso_iter != d_isogauge_items.end();++iso_iter)
		{
			specular = 0.3;
			
			color.Set(0.61,0.61,0.477);
			
			if((*iso_iter).selected)
			{
				color += TVector3(-0.2,-0.2,0.75);
				if(color.x < 0.0) color.x = 0.0;
				if(color.y < 0.0) color.y = 0.0;
				if(color.z > 1.0) color.z = 1.0;
			}
			
			if((*iso_iter).error)
			{
				color.x += 0.75;
				if(color.x > 1.0) color.x = 1.0;
			}

			if(d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face1_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face1_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face1_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face1_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face1_normal_id));
			}

			if(d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face2_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face2_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face2_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face2_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face2_normal_id));
			}

			if(d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face3_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face3_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face3_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face3_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face3_normal_id));
			}

			if(d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face4_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face4_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face4_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face4_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face4_normal_id));
			}

			if(d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face5_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face5_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face5_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face5_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face5_normal_id));
			}

			if(d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face6_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*iso_iter).face6_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*iso_iter).face6_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face6_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*iso_iter).face6_normal_id));
			}
			
			specular = 0.1;
			color.Set(0.7,0.7,0.7);

			if(d_opengl_vertex_buffer.Has_Buffer((*iso_iter).probe_stem_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*iso_iter).probe_stem_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*iso_iter).probe_stem_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*iso_iter).probe_stem_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*iso_iter).probe_stem_normal_id));
			}
						
			specular = 0.1;
			color.Set(1.0,0.0,0.0);
			
			if(d_opengl_vertex_buffer.Has_Buffer((*iso_iter).probe_ruby_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*iso_iter).probe_ruby_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*iso_iter).probe_ruby_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*iso_iter).probe_ruby_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*iso_iter).probe_ruby_normal_id));
			}
		}

		for(laser_iter = d_laser_items.begin();laser_iter != d_laser_items.end();++laser_iter)
		{
			specular = 0.3;

			if(d_opengl_vertex_buffer.Has_Buffer((*laser_iter).cube1_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*laser_iter).cube1_normal_id))
			{
				color.Set(0.477,0.677,0.477);
			
				if((*laser_iter).selected)
				{
					color += TVector3(-0.2,-0.2,0.75);
					if(color.x < 0.0) color.x = 0.0;
					if(color.y < 0.0) color.y = 0.0;
					if(color.z > 1.0) color.z = 1.0;
				}
				
				if((*laser_iter).error)
				{
					color.x += 0.75;
					if(color.x > 1.0) color.x = 1.0;
				}

				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*laser_iter).cube1_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*laser_iter).cube1_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*laser_iter).cube1_normal_id));
			}

			if(d_opengl_vertex_buffer.Has_Buffer((*laser_iter).cube2_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*laser_iter).cube2_normal_id))
			{
				color.Set(0.477,0.677,0.477);
			
				if((*laser_iter).selected)
				{
					color += TVector3(-0.2,-0.2,0.75);
					if(color.x < 0.0) color.x = 0.0;
					if(color.y < 0.0) color.y = 0.0;
					if(color.z > 1.0) color.z = 1.0;
				}
				
				if((*laser_iter).error)
				{
					color.x += 0.75;
					if(color.x > 1.0) color.x = 1.0;
				}
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*laser_iter).cube2_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*laser_iter).cube2_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*laser_iter).cube2_normal_id));
			}

			if(d_opengl_vertex_buffer.Has_Buffer((*laser_iter).shaft_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*laser_iter).shaft_normal_id))
			{
				color.Set(0.477,0.677,0.477);
				
				if((*laser_iter).error)
				{
					color.x += 0.75;
					if(color.x > 1.0) color.x = 1.0;
				}

				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*laser_iter).shaft_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*laser_iter).shaft_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*laser_iter).shaft_normal_id));
			}

			specular = 0.1;
			color.Set(0.7,0.7,0.7);

			if(d_opengl_vertex_buffer.Has_Buffer((*laser_iter).probe_z_stem_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*laser_iter).probe_z_stem_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*laser_iter).probe_z_stem_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*laser_iter).probe_z_stem_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*laser_iter).probe_z_stem_normal_id));
			}

			if(d_opengl_vertex_buffer.Has_Buffer((*laser_iter).probe_joint_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*laser_iter).probe_joint_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*laser_iter).probe_joint_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*laser_iter).probe_joint_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*laser_iter).probe_joint_normal_id));
			}

			if(d_opengl_vertex_buffer.Has_Buffer((*laser_iter).probe_xy_stem_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*laser_iter).probe_xy_stem_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*laser_iter).probe_xy_stem_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*laser_iter).probe_xy_stem_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*laser_iter).probe_xy_stem_normal_id));
			}
									
			specular = 0.1;
			color.Set(1.0,0.0,0.0);
			
			if(d_opengl_vertex_buffer.Has_Buffer((*laser_iter).probe_ruby_vertex_id) && d_opengl_vertex_buffer.Has_Buffer((*laser_iter).probe_ruby_normal_id))
			{
				
				this->Render_Triangle_Buffer(d_opengl_vertex_buffer.Buffer_Size((*laser_iter).probe_ruby_vertex_id),
											 TVector3(0,0,0),
											 color,
											 specular,
											 opacity,
											 d_opengl_vertex_buffer.Buffer((*laser_iter).probe_ruby_vertex_id),
											 d_opengl_vertex_buffer.Buffer((*laser_iter).probe_ruby_normal_id));
			}
		}
		
		d_triangle_shader_program->release();
		
		d_basic_shader_program->bind();
				
		if(d_opengl_vertex_buffer.Has_Buffer(d_machine_volume_vertex_id))
		{
			this->Render_Line_Buffer(d_opengl_vertex_buffer.Buffer_Size(d_machine_volume_vertex_id),
									 TVector3(0.5,0.5,0.5),
									 d_opengl_vertex_buffer.Buffer(d_machine_volume_vertex_id));
		}
		
		this->Render_Line_Buffer(18,TVector3(0.25,0.25,0.25),axis_lines);
		
		d_basic_shader_program->release();
		
	}
	
	
	if(!this->Is_Processing_Scale_To_Fit())
	{
		// painting of text using QPainter
		// QPainter changes one or more attributes so all relevent ones
		// are pushed then popped on end of QPainter
		glPushAttrib(GL_ACCUM_BUFFER_BIT);
		glPushAttrib(GL_VIEWPORT_BIT);
		glPushAttrib(GL_TRANSFORM_BIT);
		glPushAttrib(GL_POLYGON_BIT);
		
		glPushAttrib(GL_PIXEL_MODE_BIT);
		glPushAttrib(GL_MULTISAMPLE_BIT);
		glPushAttrib(GL_LIGHTING_BIT);
		glPushAttrib(GL_ENABLE_BIT);
		
		glPushAttrib(GL_DEPTH_BUFFER_BIT);
		glPushAttrib(GL_CURRENT_BIT);
		glPushAttrib(GL_COLOR_BUFFER_BIT);
		
		// end of openGL, start of QPainter
		QPainter							painter(this);
		
		// axis labels
		this->Point_Raster_Position(&text_x,&text_y,axis_display_positions[0]);
		text = QStringLiteral("X");
		painter.drawText(text_x,text_y,text);
		
		this->Point_Raster_Position(&text_x,&text_y,axis_display_positions[1]);
		text = QStringLiteral("Y");
		painter.drawText(text_x,text_y,text);
		
		this->Point_Raster_Position(&text_x,&text_y,axis_display_positions[2]);
		text = QStringLiteral("Z");
		painter.drawText(text_x,text_y,text);
		
		// ball bar names
		for(ballbar_iter = d_ballbar_items.begin();ballbar_iter != d_ballbar_items.end();++ballbar_iter)
		{
			this->Point_Raster_Position(&text_x,&text_y,(*ballbar_iter).label_coordinates);
			
			text = (*ballbar_iter).name;
			rect = painter.boundingRect(0,0,0,0,0,text);
			rect.moveTo(text_x,text_y);
			rect.adjust(-1,-1,2,2);
			
			painter.fillRect(rect,label_color);
			painter.drawText(rect,Qt::AlignHCenter | Qt::AlignVCenter,text);
		}
		
		painter.end();
		
		glPopAttrib();
		glPopAttrib();
		glPopAttrib();
		glPopAttrib();
		
		glPopAttrib();
		glPopAttrib();
		glPopAttrib();
		glPopAttrib();
		
		glPopAttrib();
		glPopAttrib();
		glPopAttrib();
	}
}

void TOpenGLDisplay::Render_Triangle_Buffer(
	const unsigned int 					size,
	const TVector3						&offset,
	const TVector3 						&color,
	const double 						&specular,
	const double						&opacity,
	double 								* const vertex,
	double 								* const nominal)
{
	d_triangle_shader_program->setUniformValue(d_triangle_uniform_color,color.x,color.y,color.z,opacity);
	d_triangle_shader_program->setUniformValue(d_triangle_uniform_offset,offset.x,offset.y,offset.z);
	d_triangle_shader_program->setUniformValue(d_triangle_uniform_specular,static_cast<float>(specular));
	
	glVertexAttribPointer(d_triangle_attribute_vertex, 3, GL_DOUBLE, GL_FALSE, 0, vertex);
	glVertexAttribPointer(d_triangle_attribute_normal, 3, GL_DOUBLE, GL_FALSE, 0, nominal);
	
	glEnableVertexAttribArray(d_triangle_attribute_vertex);
	glEnableVertexAttribArray(d_triangle_attribute_normal);
	
	glDrawArrays(GL_TRIANGLES, 0, size / 3);	// mode, first, index of 3 (x,y,z)
	
	glDisableVertexAttribArray(d_triangle_attribute_vertex);
	glDisableVertexAttribArray(d_triangle_attribute_normal);
}

void TOpenGLDisplay::Render_Triangle_Buffer(
	const GLenum						mode,
	const unsigned int 					size,
	const TVector3						&offset,
	const TVector3 						&color,
	const double 						&specular,
	const double						&opacity,
	double 								* const vertex,
	double 								* const nominal)
{
	d_triangle_shader_program->setUniformValue(d_triangle_uniform_color,color.x,color.y,color.z,opacity);
	d_triangle_shader_program->setUniformValue(d_triangle_uniform_offset,offset.x,offset.y,offset.z);
	d_triangle_shader_program->setUniformValue(d_triangle_uniform_specular,static_cast<float>(specular));
	
	glVertexAttribPointer(d_triangle_attribute_vertex, 3, GL_DOUBLE, GL_FALSE, 0, vertex);
	glVertexAttribPointer(d_triangle_attribute_normal, 3, GL_DOUBLE, GL_FALSE, 0, nominal);
	
	glEnableVertexAttribArray(d_triangle_attribute_vertex);
	glEnableVertexAttribArray(d_triangle_attribute_normal);
	
	glDrawArrays(mode, 0, size / 3);	// mode, first, index of 3 (x,y,z)
	
	glDisableVertexAttribArray(d_triangle_attribute_vertex);
	glDisableVertexAttribArray(d_triangle_attribute_normal);
}

void TOpenGLDisplay::Render_Line_Buffer(
	const unsigned int 					size,
	const TVector3 						&color,
	double 								* const vertex)
{
	d_basic_shader_program->setUniformValue(d_basic_uniform_color,color.x,color.y,color.z,1.0);
	
	glVertexAttribPointer(d_basic_attribute_vertex, 3, GL_DOUBLE, GL_FALSE, 0, vertex);
	
	glEnableVertexAttribArray(d_basic_attribute_vertex);
	
	glDrawArrays(GL_LINES, 0, size / 3);	// mode, first, index of 3 (x,y,z)
	
	glDisableVertexAttribArray(d_basic_attribute_vertex);
}

void TOpenGLDisplay::Render_Line_Strip_Buffer(
	const unsigned int 					size,
	const TVector3 						&color,
	double 								* const vertex)
{
	d_basic_shader_program->setUniformValue(d_basic_uniform_color,color.x,color.y,color.z,1.0);
	
	glVertexAttribPointer(d_basic_attribute_vertex, 3, GL_DOUBLE, GL_FALSE, 0, vertex);
	
	glEnableVertexAttribArray(d_basic_attribute_vertex);
	
	glDrawArrays(GL_LINE_STRIP, 0, size / 3);	// mode, first, index of 3 (x,y,z)
	
	glDisableVertexAttribArray(d_basic_attribute_vertex);
}

void TOpenGLDisplay::Render_Point_Buffer(
	const unsigned int 					size,
	const TVector3 						&color,
	double 								* const vertex)
{
	d_basic_shader_program->setUniformValue(d_basic_uniform_color,color.x,color.y,color.z,1.0);
	
	glVertexAttribPointer(d_basic_attribute_vertex, 3, GL_DOUBLE, GL_FALSE, 0, vertex);
	
	glEnableVertexAttribArray(d_basic_attribute_vertex);
	
	glDrawArrays(GL_POINTS, 0, size / 3);	// mode, first, index of 3 (x,y,z)
	
	glDisableVertexAttribArray(d_basic_attribute_vertex);
}



