/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef OPENGLDISPLAYHEADERFILE
#define OPENGLDISPLAYHEADERFILE

#include <QFrame>
#include <vector>

#include "../../core/openglwidget.h"
#include "../../core/openglvertexbuffer.h"
#include "../../core/openglgeometryfactory.h"

class QOpenGLShaderProgram;
class TOpenGLVertexBuffer;

class TOpenGLDisplay : public TOpenGLWidget
{
	Q_OBJECT
public:
	
struct TBallBar
{
	int									id;
	QString								name;
	bool								selected;
	bool								error;
	TVector3							label_coordinates;
	
	TVector3							min_xyz;
	TVector3							max_xyz;

	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> ballbar_shaft;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> ballbar_spheres;
	
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> probe_stem;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> probe_ruby;
};

struct TMCGGauge
{
	int									id;
	bool								selected;
	bool								error;
	
	TVector3							min_xyz;
	TVector3							max_xyz;

	TVector3							center;
	TVector3							axis;
	double								length;
	
};

struct TISOGauge
{
	int									id;
	bool								selected;
	bool								error;

	TVector3							min_xyz;
	TVector3							max_xyz;
	
	TVector3							axis1;
	TVector3							axis2;
	TVector3							axis3;

	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> face1;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> face2;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> face3;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> face4;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> face5;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> face6;
	
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> probe_stem;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> probe_ruby;
};
	
struct TLaser
{
	int									id;
	bool								selected;
	bool								error;

	TVector3							min_xyz;
	TVector3							max_xyz;
	
	TVector3							axis1;
	TVector3							axis2;
	TVector3							axis3;

	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> cube1;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> shaft;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> cube2;
	
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> probe_z_stem;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> probe_joint;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> probe_xy_stem;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> probe_ruby;
};

// CREATORS
	TOpenGLDisplay(const QWidget *parent=0,Qt::WindowFlags flags=Qt::WindowFlags());
	virtual ~TOpenGLDisplay(void);

// ACCESSORS

// MANIPULATORS
	
	void Define_Machine_Volume(const TVector3 &pnt_min, const TVector3 &pnt_max);
	
	void Clear_Render_Items(void);
	void Clear_Render_Volume(void);
	
	void Add_BallBar(const TBallBar &ballbar);
	void Add_MCGGauge(const TMCGGauge &mcg);
	void Add_ISOGauge(const TISOGauge &iso);
	void Add_Laser(const TLaser &laser);
	
	void Clear_Selection(void);
	void Set_Selection(const int id);
		

protected:

	virtual void GLInit(void);
	virtual void GLResize(const int Width, const int Height);
	virtual void GLRender(void);

private:
	
struct TRenderMCGItem
{
	int									id;
	bool								selected;
	bool								error;
	
	int									mcg_vertex_id;
	int									mcg_nominal_id;
	GLenum								mode;
	TVector3							color;
};
	
struct TRenderBallbarItem
{
	int									id;
	QString								name;
	bool								selected;
	bool								error;
	TVector3							label_coordinates;
	
	int									ballbar_shaft_vertex_id;
	int									ballbar_sphere_vertex_id;
	int									ballbar_shaft_normal_id;
	int									ballbar_sphere_normal_id;
	
	int									probe_stem_vertex_id;
	int									probe_ruby_vertex_id;
	int									probe_stem_normal_id;
	int									probe_ruby_normal_id;
};
	
struct TRenderISOItem
{
	int									id;
	bool								selected;
	bool								error;
	
	int									face1_vertex_id;
	int									face2_vertex_id;
	int									face3_vertex_id;
	int									face4_vertex_id;
	int									face5_vertex_id;
	int									face6_vertex_id;

	int									face1_normal_id;
	int									face2_normal_id;
	int									face3_normal_id;
	int									face4_normal_id;
	int									face5_normal_id;
	int									face6_normal_id;

	int									probe_stem_vertex_id;
	int									probe_ruby_vertex_id;
	int									probe_stem_normal_id;
	int									probe_ruby_normal_id;
};

struct TRenderLaserItem
{
	int									id;
	bool								selected;
	bool								error;
	
	int									cube1_vertex_id;
	int									shaft_vertex_id;
	int									cube2_vertex_id;

	int									cube1_normal_id;
	int									shaft_normal_id;
	int									cube2_normal_id;

	int									probe_z_stem_vertex_id;
	int									probe_xy_stem_vertex_id;
	int									probe_joint_vertex_id;
	int									probe_ruby_vertex_id;
	
	int									probe_z_stem_normal_id;
	int									probe_xy_stem_normal_id;
	int									probe_joint_normal_id;
	int									probe_ruby_normal_id;
};

	TVector3							d_machine_min;
	TVector3							d_machine_max;
	TVector3							d_axis_draw_pos;
	double								d_axis_draw_len;
	
	TOpenGLVertexBuffer					d_opengl_vertex_buffer;
	
	QOpenGLShaderProgram				*d_triangle_shader_program;
	QOpenGLShaderProgram				*d_basic_shader_program;

	GLint								d_triangle_uniform_color;
	GLint								d_triangle_uniform_specular;
	GLint								d_triangle_uniform_offset;
	GLint								d_triangle_attribute_vertex;
	GLint								d_triangle_attribute_normal;
	
	GLint								d_basic_uniform_color;
	GLint								d_basic_attribute_vertex;

	int									d_machine_volume_vertex_id;
	
	std::vector<TRenderBallbarItem>		d_ballbar_items;
	std::vector<TRenderMCGItem>			d_mcg_base_items;
	std::vector<TRenderMCGItem>			d_mcg_arm_items;
	std::vector<TRenderISOItem>			d_isogauge_items;
	std::vector<TRenderLaserItem>		d_laser_items;
	
	void Render_Triangle_Buffer(const unsigned int size,const TVector3 &offset,const TVector3 &color, const double &specular,const double &opacity, double * const vertex,double * const nominal);
	void Render_Triangle_Buffer(const GLenum mode,const unsigned int size,const TVector3 &offset,const TVector3 &color, const double &specular,const double &opacity, double * const vertex,double * const nominal);	void Render_Line_Buffer(const unsigned int size,const TVector3 &color, double * const vertex);
	void Render_Line_Strip_Buffer(const unsigned int size,const TVector3 &color, double * const vertex);
	void Render_Point_Buffer(const unsigned int size,const TVector3 &color, double * const vertex);


// NOT IMPLEMENTED
	TOpenGLDisplay& operator=(const TOpenGLDisplay&);
};

#endif

