/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QHBoxLayout>
#include <QProgressBar>
#include <QPushButton>
#include <QSpacerItem>
#include <QVBoxLayout>

#include "progressdialog.h"

TProgressDialog::TProgressDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QHBoxLayout							*button_hlayout;
	QPushButton							*cancel_button;
	QSpacerItem							*button_hspacer;
	QVBoxLayout							*dialog_vlayout;

	this->resize(283,84);

	dialog_vlayout = new QVBoxLayout(this);

	d_progress_bar = new QProgressBar(this);
	d_progress_bar->setValue(24);
	dialog_vlayout->addWidget(d_progress_bar);

	button_hlayout = new QHBoxLayout();

	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	cancel_button = new QPushButton(this);
	button_hlayout->addWidget(cancel_button);
	dialog_vlayout->addLayout(button_hlayout);

	this->setWindowTitle(QStringLiteral("Progress"));
	cancel_button->setText(QStringLiteral("Cancel"));

	connect(cancel_button,&QPushButton::clicked,this,&TProgressDialog::Cancel);
}

TProgressDialog::~TProgressDialog(void)
{
}

void TProgressDialog::Reset_Progress(
	const int							min,
	const int							max)
{
	d_progress_bar->setRange(min,max);
	d_progress_bar->setValue(min);
}

void TProgressDialog::Set_Progress(
	const int							value)
{
	d_progress_bar->setValue(value);
}

void TProgressDialog::End_Progress(void)
{
	d_progress_bar->setValue(d_progress_bar->minimum());
}
