/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef TYPESHEADERFILE
#define TYPESHEADERFILE

#include <QString>

namespace Types
{
	static const int					AXIS_X								= 0x00000001;
	static const int					AXIS_Y								= 0x00000002;
	static const int					AXIS_Z								= 0x00000004;
	static const int					AXIS_D								= 0x00000008;
	static const int					AXIS_XYZ							= 0x00000010;

	static const int					KINEMATIC_XYZ						= 0x00000100;
	static const int					KINEMATIC_YXZ						= 0x00000200;
	static const int					KINEMATIC_XZY						= 0x00000400;
	static const int					KINEMATIC_YZX						= 0x00000800;

	static const int					PARAMETER_RX						= 0x00001000;
	static const int					PARAMETER_RY						= 0x00002000;
	static const int					PARAMETER_RZ						= 0x00004000;
	static const int					PARAMETER_SCALE						= 0x00008000;
	static const int					PARAMETER_DEFLECTION				= 0x00010000;
	static const int					PARAMETER_QXY						= 0x00020000;
	static const int					PARAMETER_QYZ						= 0x00040000;
	static const int					PARAMETER_QZX						= 0x00080000;
	
enum TMeasurementType
{
	MEASUREMENT_ISO = 0,
	MEASUREMENT_LASER,
	MEASUREMENT_MCG,
	MEASUREMENT_BALLBAR
};

};

class TMachineAxisException
{
public:
	
// CREATORS
	TMachineAxisException(const QString &error);
	~TMachineAxisException(void);
	
// ACCESSORS
	QString Error_Text(void) const {return d_error_text;}
	
private:
	QString							d_error_text;
};


#endif
